#### check the rrt data for inconsistencies

rm(list=ls())

require(lubridate)

source("code/data_prep/scripts/C000_helpfun.R")
source("code/data_prep/scripts/C005_helpfun_ref.R")

path <- "code/data_prep/daymeans/sef/"
outpath <- "code/data_prep/daymeans/histseries/homref"

inventory <- readWorksheetFromFile("code/data_input/arm_stations_inventory_withnbcn.xlsx",sheet=1, header=T,colTypes="character",startRow=2)

files <- list.files(path, "rr0", full.names = T)
sef <- lapply(files, read_sef, all=T)
id <- sapply(files, function(x){read_meta(x)["id"]})
meta <- lapply(files, read_meta)
names(sef) <- id

### create TOT
daterange <- seq(as.Date("1761-01-01"),as.Date("1863-12-31"),by="day")
dattot <- as.data.frame(matrix(NA, ncol=length(id)+1, nrow=length(daterange)))
dattot[,1] <- daterange

for(ff in 1:length(sef)){
  print(id[ff])
  dindm <- daterange %in% pdate(sef[[ff]][,c("Year","Month","Day")])
  dinds<-  pdate(sef[[ff]][,c("Year","Month","Day")]) %in% daterange
  dattot[dindm,ff+1] <- sef[[ff]]$Value[dinds]
}

colnames(dattot) <- c("date", id)

datesum <- apply(dattot[,2:ncol(dattot)], 1, function(x){ sum(!is.na(x))})
png(paste0("code/data_prep/plots/rrt/hist_period_data_availability_daily",Sys.Date(),".png"), width = 7800, height=800)
barplot(datesum)
dev.off()

wetdayfreq <- sapply(2:ncol(dattot), function(x){
  val <- aggregate(dattot[,x], list(substr(dattot$date,1,7)),
                   function(y){
                     y <- y[!is.na(y)]
                     sum(y)/length(y)})$x
  return(val)
  })

colnames(wetdayfreq) <- id
ymons <- unique(substr(dattot$date,1,7))
rownames(wetdayfreq) <- ymons


apply(wetdayfreq, 2,function(x){all(x[!is.na(x)]==1)})

stncols <- c("black","blue3","cornflowerblue","turquoise1","forestgreen","green","yellowgreen","yellow","orange",
             "darkred","tomato","pink", "purple","magenta")

png(paste0("code/data_prep/plots/rrt/hist_period_wetdayfreq",Sys.Date(),".png"), width = 2800, height=800, res=180)
par(mfrow=c(2,1), mar=c(2,2,2,10))
plot(NA, xlim=c(1,nrow(wetdayfreq)/2), ylim=c(0,1), xaxt="n")
early <- apply(wetdayfreq[1:(nrow(wetdayfreq)/2),],2, function(x){!all(is.na(x))})
help <- which(early)
lapply(1:length(help), function(x){points(1:nrow(wetdayfreq), wetdayfreq[,help[x]], type="l", col=stncols[x])})
axis(side=1,at=seq(1,nrow(wetdayfreq),12*5), labels = rownames(wetdayfreq)[seq(1,nrow(wetdayfreq),12*5)])
par(xpd=TRUE)
legend("right", inset=c(-0.12,0), legend=id[help], col=stncols[1:length(help)],pch=20, bty="n", cex=0.7)
par(xpd=F)
plot(NA, xlim=c(nrow(wetdayfreq)/2,nrow(wetdayfreq)), ylim=c(0,1), xaxt="n")
help <- c(which(!early), grep("Aarau",names(early)))
lapply(1:length(help), function(x){points(1:nrow(wetdayfreq), wetdayfreq[,help[x]], type="l", col=stncols[x])})
axis(side=1,at=seq(1,nrow(wetdayfreq),12*5), labels = rownames(wetdayfreq)[seq(1,nrow(wetdayfreq),12*5)])
par(xpd=TRUE)
legend("right", inset=c(-0.12,0), legend=id[help], col=stncols[1:length(help)],pch=20, bty="n", cex=0.7)
par(xpd=F)
dev.off()

### read ref series for comparison
rr0files_ref <- list.files(pattern="_rr0_",path="code/data_prep/daymeans/refseries/hom/",full.names = T)
rr0sef_ref <- lapply(rr0files_ref, read_sef, all=T)
rr0meta_ref <- lapply(rr0files_ref, read_meta)
id_ref <- sapply(rr0meta_ref, function(x)x["id"])

### create TOT
daterange_ref <- seq(as.Date("1961-01-01"),as.Date("2020-12-31"),by="day")
dattot_ref <- as.data.frame(matrix(NA, ncol=length(id_ref)+1, nrow=length(daterange_ref)))
dattot_ref[,1] <- daterange_ref

for(ff in 1:length(rr0files_ref)){
  print(id_ref[ff])
  dindm <- daterange_ref %in% pdate(rr0sef_ref[[ff]][,c("Year","Month","Day")])
  dinds<-  pdate(rr0sef_ref[[ff]][,c("Year","Month","Day")]) %in% daterange_ref
  dattot_ref[dindm,ff+1] <-rr0sef_ref[[ff]]$Value[dinds]
}

colnames(dattot_ref) <- c("date", id_ref)

wetdayfreq_ref <- sapply(2:ncol(dattot_ref), function(x){
  val <- aggregate(dattot_ref[,x], list(substr(dattot_ref$date,1,7)),
                   function(y){
                     y <- y[!is.na(y)]
                     sum(y)/length(y)})$x
  return(val)
})

colnames(wetdayfreq_ref) <- id_ref

## compare the wetday freq to all other stations for every station individually

for(stn in 1:length(id_ref)){
  png(paste0("code/data_prep/plots/rrt/allindiv/wetdayfreq_ref_",id_ref[stn],"_",Sys.Date(),".png"), width = 900, height = 300)
  plot(1:nrow(wetdayfreq_ref), wetdayfreq_ref[,stn], type="l", col="black", xaxt="n")
  abline(h=seq(0,1,0.2), lty=3, col="gray60")
  axis(side=1, at=seq(1,nrow(wetdayfreq_ref),12),labels = c(1961:2020))
  title(id_ref[stn])
  dev.off()
}

for(stn in 1:length(id)){
  if(!all(is.na(wetdayfreq[,stn]))){
    png(paste0("code/data_prep/plots/rrt/allindiv/wetdayfreq_hist_",id[stn],"_",Sys.Date(),".png"), width = 1490, height = 300)
    plot(1:nrow(wetdayfreq), wetdayfreq[,stn], type="l", col="black", xaxt="n")
    abline(h=seq(0,1,0.2), lty=3, col="gray60")
    axis(side=1,at=seq(1,nrow(wetdayfreq),12*5), labels = rownames(wetdayfreq)[seq(1,nrow(wetdayfreq),12*5)])
    title(id[stn])
    dev.off()
  }
}

  
  


stncols <- c("black","blue3","cornflowerblue","turquoise1","forestgreen","green","yellowgreen","yellow","orange",
             "darkred","tomato","pink", "purple","magenta")

early <- apply(wetdayfreq[1:(nrow(wetdayfreq)/2),],2, function(x){!all(is.na(x))})
plot(NA, xlim=c(nrow(wetdayfreq)/2,nrow(wetdayfreq)), ylim=c(0,1))
help <- c(which(!early))
lapply(1:length(help), function(x){points(1:nrow(wetdayfreq), wetdayfreq[,help[x]], type="l", col=stncols[x])})
abline(h=0.4, lty=3, col="gray65")
par(xpd=TRUE)
legend("right", inset=c(-0.16,0), legend=id[help], col=stncols[1:length(help)],pch=20, bty="n", cex=0.7)
par(xpd=F)
plot(NA, xlim=c(1,nrow(wetdayfreq_ref)), ylim=c(0,1), xaxt="n")
axis(side=1, at=seq(1,nrow(wetdayfreq_ref),12),labels = c(1961:2020))
lapply(grep("ELL|SMA",id_ref), function(x){points(1:nrow(wetdayfreq_ref), wetdayfreq_ref[,x], type="l", col=stncols[x])})
abline(h=0.4, lty=3, col="gray65")
par(xpd=TRUE)
legend("right", inset=c(-0.12,0), legend=id_ref[grep("ELL|SMA",id_ref)], col=stncols[grep("ELL|SMA",id_ref)],pch=20, bty="n", cex=0.7)
par(xpd=F)

#################################

par(mar=c(2,2,2,10))
plot(NA, xlim=c(1,nrow(wetdayfreq)), ylim=c(0,1), xaxt="n")
axis(side=1, at=seq(1,nrow(wetdayfreq),12),labels = c(year(daterange)[1]:year(daterange[length(daterange)])))
lapply(grep("Ellikon|Wint",id), function(x){points(1:nrow(wetdayfreq), wetdayfreq[,x], type="l", col=stncols[x])})
par(xpd=TRUE)
legend("right", inset=c(-0.12,0), legend=id, col=stncols[1:length(id)],pch=20, bty="n", cex=0.7)
par(xpd=F)


corrmat <- round(cor(wetdayfreq, use="pairwise.complete.obs"),3)
png(paste0("code/data_prep/plots/rrt/cormat_wetdayfreq",Sys.Date(),".png"), width = 1200, height = 1200, res=180)
par(mar=c(12,12,2,2))
cols <- colorRampPalette(RColorBrewer::brewer.pal(n=11-1,"Reds"))(11-1)
fields::image.plot(1:ncol(corrmat),1:ncol(corrmat),corrmat, col=cols, breaks=seq(0,1,0.1),ylab="",xlab="", xaxt="n", yaxt="n")
axis(side=1, las=2, labels = id, at=seq(1:ncol(corrmat)))
axis(side=2, las=2,labels = id, at=seq(1:ncol(corrmat)))
abline(v=1:ncol(corrmat), lty=2, col="gray65", lwd=0.5)
abline(h=1:ncol(corrmat), lty=2, col="gray65", lwd=0.5)
dev.off()

#### merge long series and keep only those with sufficient quality, defined visually.

zurifiles <- grepl("ZH01|Zurich", id)
origfilename <- inventory[,"orig_filename"]

for(ll in which(!zurifiles)){
  
  if(!grepl("Wint|Fri",id[ll])){
    print(id[ll])
    ss <- sef[[ll]]
    mm <- meta[[ll]]
    fff <- gsub(".*sef//","",files[ll])
    newcode <- substr(inventory[grepl(fff, origfilename),"ID"],1,3)
    
    write_sef(Data = ss[, c(2:6,8)],
              outpath = outpath,
              variable = "rr0",
              cod = newcode,
              nam = mm["name"],
              lat = mm["lat"],
              lon = mm["lon"],
              alt = mm["alt"],
              link = "",
              sou = "WeaR",
              units = "coded",
              stat = mm["stat"],
              metaHead = "",
              meta=ss$Meta,
              period = "day",
              time_offset =0,
              keep_na = TRUE,
              note="daily")
  }

}

##### zurich

yrr <- range(unlist(lapply(sef[which(zurifiles)], function(x) {range(x$Year)})))
daterange <- seq(as.Date(paste0(yrr[1],"-01-01")),as.Date(paste0(yrr[2],"-12-31")), by="day")
sefdata <- data.frame(year=year(daterange),
                      mo=month(daterange),
                      d=day(daterange),
                      h=rep(NA, length(daterange)),
                      m=rep(NA,length(daterange)),
                      Value=rep(NA,length(daterange)),
                      Meta=rep(NA,length(daterange)))
id[zurifiles]

ottind <- which(daterange %in% pdate(sef$ZH01_Zurich_Ott[,c(2:4)]))
ulrind <- which(daterange %in% pdate(sef$ZH01_Zurich_Ulrich[,c(2:4)]))
feeind <- which(daterange %in% pdate(sef$GCOS_Zurich_Feer[,c(2:4)]))
horind <- setdiff(which(daterange %in% pdate(sef$GCOS_Zurich_Horner[,c(2:4)]) ),feeind)
horind2 <- which(pdate(sef$GCOS_Zurich_Horner[,c(2:4)]) %in% daterange[horind])
murind <- which(daterange %in% pdate(sef$GCOS_Zurich_Muralt[,c(2:4)]))
sngind <- which(daterange %in% pdate(sef$GCOS_Zurich_SNG[,c(2:4)]))

sefdata[ottind,"Value"] <- sef$ZH01_Zurich_Ott$Value
sefdata[ulrind,"Value"] <- sef$ZH01_Zurich_Ulrich$Value
sefdata[feeind,"Value"] <- sef$GCOS_Zurich_Feer$Value
sefdata[horind,"Value"] <- sef$GCOS_Zurich_Horner$Value[horind2]
sefdata[murind,"Value"] <- sef$GCOS_Zurich_Muralt$Value
sefdata[sngind,"Value"] <- sef$GCOS_Zurich_SNG$Value

sefdata[ottind,"Meta"] <- sef$ZH01_Zurich_Ott$Meta
sefdata[ulrind,"Meta"] <- sef$ZH01_Zurich_Ulrich$Meta
sefdata[feeind,"Meta"] <- sef$GCOS_Zurich_Feer$Meta
sefdata[horind,"Meta"] <- sef$GCOS_Zurich_Horner$Meta[horind2]
sefdata[murind,"Meta"] <- sef$GCOS_Zurich_Muralt$Meta
sefdata[sngind,"Meta"] <- sef$GCOS_Zurich_SNG$Meta


write_sef(Data = sefdata[, 1:7],
          outpath = outpath,
          variable = "rr0",
          cod = "SMA",
          nam = "Zurich",
          lat = 47.371,
          lon = 8.534,
          alt = 420,
          link = "",
          sou = "WeaR",
          units = "coded",
          stat = "",
          metaHead = "merged from different souces CHIMES|GCOS",
          meta=sefdata$Meta,
          period = "day",
          time_offset =0,
          keep_na = TRUE,
          note="daily")




