########################################################
############## create standard format for ref-series
########################################################

rm(list=ls())

require(dataresqc)
require(geosphere)
require(ncdf4)
require(XLConnect)
require(lubridate)
require(qmap)

source("code/data_prep/scripts/Cget_ncdf.R")
source("code/data_prep/scripts/C005_helpfun_ref.R")
source("code/data_prep/scripts/C000_helpfun.R")

inventory <- XLConnect::readWorksheetFromFile("code/data_input/arm_stations_inventory_withnbcn.xlsx",sheet=1, header=T,colTypes="character",startRow=2)
refper <- c(1961,2020)
sef <- "code/data_prep/daymeans/sef/"
rhires <- "code/data_input/RhiresD_ch01h.swiss.lv95_19610101-20201231.nc"
outpath <- "code/data_prep/daymeans/refseries/hom/"

rr_th <- 1

########################################
#### rr0
########################################
varind <- which(inventory$var%in%c("rrt") & inventory$no!="no" & inventory$no!="ind")
refstn <- inventory[varind,"reference_station_data"]
inventory[varind,"ID"]

for(ff in 1:length(varind)){
  
  print(refstn[ff])
  (ref <- stringr::str_sub(unlist(strsplit(refstn[ff],",")), end=3))
  
  if(!is.na(ref) & inventory$no[ff]!="no"){
    var <- inventory[varind[ff],"var"]
    id <- inventory[varind[ff],"ID"]
    xlat <- as.numeric(inventory[varind[ff],"lat_hist"])
    xlon <- as.numeric(inventory[varind[ff],"lon_hist"])
    
    ## 1) if reference data is from mch
    if(ref=="mch"){
      refhelp <- unlist(strsplit(refstn[ff],","))
      dat <- read.table(paste0(refpath,refhelp), header = T, skip=2,na.strings = "-", sep=";")
      dat <- dat[dat$stn==gsub("_rr","",id),]
      meta <- read.table(paste0(refpath,gsub("data","legend",refstn[ff])),skip=31,nrow=6,
                         colClasses = rep("character",7),strip.white=T)
      meta <- meta[meta$V1==gsub("_rr","",id),]
      
      ## https://rdrr.io/github/falling-fruit/fruitr/man/ch1903_to_wgs84.html
      ch <- fruitr::ch1903_to_wgs84(as.integer(c(substr(meta["V6"],1,6),substr(meta["V6"],8,14))))
      xlon <- ch[1]
      xlat <- ch[2]
      
      out <- data.frame(Year=as.integer(substr(dat$time,1,4)),
                        Month=as.integer(substr(dat$time,5,6)),
                        Day=as.integer(substr(dat$time,7,8)),
                        Hour=rep(NA, nrow(dat)),
                        Minute=rep(NA, nrow(dat)),
                        Value=as.numeric(as.character(dat[,"rhs150d0"])),
                        Meta=rep("mchstn",nrow(dat)))
      out <- check_dates(out)
      
      # if(inventory[varind[ff],"gap_filling_data"]!="no"){
      #   out <- fill_gaps(out,gap_data=inventory[varind[ff],"gap_filling_data"],xlon=xlon,xlat=xlat,var=var,id=id) 
      #   }
      refind <- which(out$Year %in% c(refper[1]:refper[2]))
      out <- out[refind,]
      
      write_sef(out[,1:7], outpath, "rr0", paste0(inventory[varind[ff],"ID"],"_ref"), meta["V1"], 
                xlat, xlon, meta["V7"], "MCH", "idaweb", units="mm", "sumn", period="day", meta = out$Meta, note="daily", keep_na=TRUE)
      
    }
    ## 2) if reference from rhires
    if(ref=="rhi"){
      dat <- list()
      ncp <- nc_open(rhires)
      lons <- ncvar_get(ncp, "lon")
      lats <- ncvar_get(ncp, "lat")
      coords <- data.frame(lon=as.vector(lons), lat=as.vector(lats))
      
      ## Select closest grid point on land (limit the search to a square of 25 grid points)       
      i_grid <- which.min(distGeo(c(xlon,xlat), coords))
      i_lonlat <- which(lons == coords$lon[i_grid], arr.ind = T)
      # i_lon <- which(lons == coords$lon[i_grid])
      # i_lat <- which(lats == coords$lat[i_grid])
      dat$dat <- ncvar_get(ncp, "RhiresD", c(i_lonlat[1],i_lonlat[2],1), c(1,1,-1))
      dat$time <- nc_time_p(ncp,timename = "time")
      
      out <- data.frame(Year=lubridate::year(dat$time),
                        Month=lubridate::month(dat$time),
                        Day=lubridate::day(dat$time),
                        Hour=rep(NA, nrow(dat$dat)),
                        Minute=rep(NA, nrow(dat$dat)),
                        Value=round(dat$dat,2),
                        Meta=rep("RhiresD",nrow(dat$dat)))
      out <- check_dates(out)
    
      ### convert to rr0
      out$Value <- ifelse(out$Value > rr_th,1,0)
      
      refind <- which(out$Year %in% c(refper[1]:refper[2]))
      out <- out[refind,]
      
      write_sef(out[,1:7], outpath, "rr0", paste0(inventory[varind[ff],"ID"],"_ref"), inventory[varind[ff],"Name"], 
                lat=xlat, lon=xlon, inventory[varind[ff],"alt_hist"], sou="WeaR", link="",
                units ="coded", stat="sum", period="day", metaHead = "",meta=out$Meta, note="nothom", keep_na=TRUE)
      
    }
  }
  
}


#### calc wet day frequency as comparison for hist data
rr0files <- list.files(pattern="_rr0_",path=outpath,full.names = T)
rr0sef <- lapply(rr0files, read_sef, all=T)
rr0meta <- lapply(rr0files, read_meta)
id <- sapply(rr0meta, function(x)x["id"])

### create TOT
daterange <- seq(as.Date(paste0(refper[1],"-01-01")),as.Date(paste0(refper[2],"-01-01")),by="day")
dattot <- as.data.frame(matrix(NA, ncol=length(id)+1, nrow=length(daterange)))
dattot[,1] <- daterange

for(ff in 1:length(rr0files)){
  print(id[ff])
  dindm <- daterange %in% pdate(rr0sef[[ff]][,c("Year","Month","Day")])
  dinds<-  pdate(rr0sef[[ff]][,c("Year","Month","Day")]) %in% daterange
  dattot[dindm,ff+1] <-rr0sef[[ff]]$Value[dinds]
}

colnames(dattot) <- c("date", id)

wetdayfreq <- sapply(2:ncol(dattot), function(x){
  val <- aggregate(dattot[,x], list(substr(dattot$date,1,7)),
                   function(y){
                     y <- y[!is.na(y)]
                     sum(y)/length(y)})$x
  return(val)
})

colnames(wetdayfreq) <- id

apply(wetdayfreq, 2,function(x){all(x[!is.na(x)]==1)})

stncols <- c("black","blue3","cornflowerblue","turquoise1","forestgreen","green","yellowgreen","yellow","orange",
             "darkred","tomato","pink", "purple","magenta")

png(paste0("code/data_prep/plots/rrt/ref_period_wetdayfreq",Sys.Date(),".png"), width = 2900, height=800, res=180)
par(mar=c(2,2,2,10))
plot(NA, xlim=c(1,nrow(wetdayfreq)), ylim=c(0,1), xaxt="n")
axis(side=1, at=seq(1,nrow(wetdayfreq),12),labels = c(refper[1]:refper[2]))
lapply(1:ncol(wetdayfreq), function(x){points(1:nrow(wetdayfreq), wetdayfreq[,x], type="l", col=stncols[x])})
par(xpd=TRUE)
legend("right", inset=c(-0.12,0), legend=id, col=stncols[1:length(id)],pch=20, bty="n", cex=0.7)
par(xpd=F)
dev.off()


par(mar=c(2,2,2,10))
plot(NA, xlim=c(1,nrow(wetdayfreq)), ylim=c(0,1), xaxt="n")
axis(side=1, at=seq(1,nrow(wetdayfreq),12),labels = c(refper[1]:refper[2]))
lapply(grep("ELL|WIN",id), function(x){points(1:nrow(wetdayfreq), wetdayfreq[,x], type="l", col=stncols[x])})
par(xpd=TRUE)
legend("right", inset=c(-0.12,0), legend=id, col=stncols[1:length(id)],pch=20, bty="n", cex=0.7)
par(xpd=F)


