########################################################
############## create standard format for ref-series
########################################################
#  all to sef, including info in meta, where other datasets have been used
# https://rmets.onlinelibrary.wiley.com/doi/full/10.1002/joc.4870 paper for swiss bias adjustments
# formula for pressure conversion to station altitude

rm(list=ls())

require(dataresqc)
require(geosphere)
require(ncdf4)
require(XLConnect)
require(lubridate)
require(qmap)

source("code/data_prep/scripts/Cget_ncdf.R")
source("code/data_prep/scripts/C005_helpfun_ref_new.R")
source("code/data_prep/scripts/C000_helpfun.R")

inventory <- XLConnect::readWorksheetFromFile("code/data_input/arm_stations_inventory_withnbcn.xlsx",sheet=1, header=T,colTypes="character",startRow=2)
refper <- c(1961,2020)
#sef <- "code/data_prep/daymeans/hom/"
refpath <- "data/"
eobs_ta <- "eobs/tg_ens_mean_3154052_v23.nc"
tabs <- "data/tabs/TabsD_ch01r.swiss.lv95_19610101-20201231.nc"
outpath <- "code/data_prep/daymeans/refseries/sef/"

########################################
#### Temperatur
########################################
varind <- which(inventory$var%in%c("ta") & !grepl("no|ind|luc", inventory$no))
refstn <- inventory[varind,"reference_station_data"]
inventory$ID[varind]

ff <- 10

for(ff in 1:length(varind)){
#for(ff in 6:10){
  print(ff)  
  print(refstn[ff])
  (ref <- stringr::str_sub(unlist(strsplit(refstn[ff],",")), end=3))
  
  if(!is.na(ref[1])){
  var <- inventory[varind[ff],"var"]
  (id <- inventory[varind[ff],"ID"])
  xlat <- as.numeric(inventory[varind[ff],"lat_hist"])
  xlon <- as.numeric(inventory[varind[ff],"lon_hist"])
  alt <- as.numeric(inventory[varind[ff],"alt_hist"])
  
  ## 0) several reference datasets
  if(length(ref)>1){
    
    ## read the entire dataset at that grid point
    outl <- list()
    metavals <- list()

    for(i in 1:length(ref)){
      ii <- ref[i]
      print(ii)
      ## read in full period in order to do qm
       if(ii=="eob"){
        outl[[i]] <- read_eobs(path=get(paste0("eobs_",var)),xlon=xlon,xlat=xlat,var=var)
        outl[[i]][,2:ncol(outl[[i]])] <- check_dates(outl[[i]][,2:ncol(outl[[i]])])
      }
      if(ii=="tab"){
        dat <- list()
        ncp <- nc_open(tabs)
        lons <- ncvar_get(ncp, "lon")
        lats <- ncvar_get(ncp, "lat")
        coords <- data.frame(lon=rep(lons,each=length(lats)), lat=rep(lats,times=length(lons)))
        
        ## get coordinates of station
        xlon <- as.numeric(inventory[varind[ff],"lon_hist"])
        xlat <- as.numeric(inventory[varind[ff],"lat_hist"])
        
        ## Select closest grid point on land (limit the search to a square of 25 grid points)       
        i_grid <- which.min(distGeo(c(xlon,xlat), coords))
        i_lon <- which(lons == coords$lon[i_grid])
        i_lat <- which(lats == coords$lat[i_grid])
        dat$dat <- ncvar_get(ncp, "TabsD", c(i_lon,i_lat,1), c(1,1,-1))
        dat$time <- nc_time_p(ncp,timename="REFERENCE_TS")
        
        outl[[i]] <- data.frame(Var=rep("ta", nrow(dat$dat)),
                                Year=lubridate::year(dat$time),
                                Month=lubridate::month(dat$time),
                                Day=lubridate::day(dat$time),
                                Hour=rep(24, nrow(dat$dat)),
                                Minute=rep(0, nrow(dat$dat)),
                                Value=dat$dat,
                                Meta=rep("TabsD",nrow(dat$dat)))
        outl[[i]][,2:ncol(outl[[i]])] <- check_dates(outl[[i]][,2:ncol(outl[[i]])])
      }
      if(ii=="ECA"){
        refhelp <- unlist(strsplit(refstn[ff],","))[i]
        dat <- read.table(paste0(refpath,"ecad/",refhelp), header = T, skip=20, sep=",", na.strings = -9999)
        meta <- read.table(paste0(refpath,"ecad/",gsub("/.*","",refhelp),"/stations.txt"),
                           skip=19,sep=",",colClasses = rep("character",6), strip.white=T, allowEscapes=T)
        realend <- which(sapply(1:nrow(dat),function(i) all(is.na(dat[i:nrow(dat),4]))))[1]
        
        dat <- dat[1:(realend-1),]
        meta <- meta[which(meta$V1==dat$STAID[1]),]
        lon  <- meta["V5"]
        lat  <- meta["V4"]
        years <- as.integer(substr(dat$DATE,1,4))

        outl[[i]] <- data.frame(Var=rep("ta", nrow(dat)),
                                Year=years,
                                Month=as.integer(substr(dat$DATE,5,6)),
                                Day=as.integer(substr(dat$DATE,7,8)),
                                Hour=rep(NA, nrow(dat)),
                                Minute=rep(NA, nrow(dat)),
                                Value=dat[,switch(var,"p"="PP","ta"="TG")]*0.1,
                                Meta=rep("ECA-qced",nrow(dat)))
        outl[[i]]$Value[which(dat[,paste0("Q_",switch(var,"p"="PP","ta"="TG"))] %in% c(1,9))] <- NA
        outl[[i]][,2:ncol(outl[[i]])] <- check_dates(outl[[i]][,2:ncol(outl[[i]])])
      }
      if(ii=="Wea"){
        refhelp <- unlist(strsplit(refstn[ff],","))[i]
        dat <- read_sef(paste0("code/data_prep/daymeans/hom/",refhelp), all=T)
        meta <- read_meta(paste0("code/data_prep/daymeans/hom/",refhelp))
        years <- dat$Year

        outl[[i]] <- data.frame(Var=rep("ta", nrow(dat)),
                                Year=years,
                                Month=dat$Month,
                                Day=dat$Day,
                                Hour=dat$Hour,
                                Minute=dat$Minute,
                                Value=dat$Value,
                                Meta=dat$Meta)
        outl[[i]][,2:ncol(outl[[i]])] <- check_dates(outl[[i]][,2:ncol(outl[[i]])])
      }
      if(ii=="eca"){ ## this is the homogenized ecad data from EUSTACe/YURI
        refhelp <- unlist(strsplit(refstn[ff],","))[i]
        hhelp <- list.files("data/ecad/", pattern=refhelp, full.names = T)
        dat <- load_object(hhelp)
        stnind <- grepl(stringr::str_sub(id, end= -4),dat$data_info$pnames)
        len <- ncol(dat$data$tavg)
        realend <- which(sapply(1:len,function(i) all(is.na(dat$data$tavg[stnind,i:len]))))[1]-1
        start <- which(year(dat$time) > 1949)[1]
        
        outl[[i]] <- data.frame(Var=rep("ta", length(start:realend)),
                                Year=year(dat$time[start:realend]),
                                Month=month(dat$time[start:realend]),
                                Day=day(dat$time[start:realend]),
                                Hour=rep(NA,length(start:realend)),
                                Minute=rep(NA,length(start:realend)),
                                Value=round(dat$data$tavg[stnind,start:realend],2),
                                Meta=rep("ecad",length(start:realend)))
        outl[[i]][,2:ncol(outl[[i]])] <- check_dates(outl[[i]][,2:ncol(outl[[i]])])
      }
    }
    
    names(outl) <- ref
    adj <- list()
    if(any(ref %in% c("eob","tab"))){
      ## write back prediction for grid, only after break-year
      ## Padua temperature has basically no overlap
      if(length(ref)==3){
        grind <- which(ref%in%c("ECA"))
        helpind <- which(pdate(outl[[grind]][,2:4]) > tail(pdate(outl[[1]][,2:4]),1))
        outl[[1]] <- rbind(outl[[1]],outl[[grind]][helpind,] )
        outl <- outl[c(1,3)]
        ref <- c("eca","eob")
      }
      print("adjust grid")
      adj <- adjust_grid_to_station(outl=outl,var=var,id=id)
      outl[[which(!ref%in%c("mch","ECA","Wea","eca"))]] <- adj$outn
    }

    out <- rbind(outl[[1]],outl[[2]])
    refind <- pdate(out[,2:4])>= as.Date(paste0(refper[1],"-01-01")) & pdate(out[,2:4])<= as.Date(paste0(refper[2],"-12-31"))
    out <- out[refind,]
  
  write_sef(out[,2:8], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), paste0("merged_",var), 
            xlat, xlon, inventory[varind[ff],"alt_hist"], sou=toupper(cllps(ref,"-")),
            link="various", 
            units="C", stat="mean", meta=out$Meta,
            period="day",  note="daily_merged", keep_na=TRUE)
  
  ## make a qc
  file <- list.files(outpath, pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
  qc(file, outpath=paste0("code/data_prep/daymeans/refseries/qcfiles/"))
  qcf <- list.files(paste0("code/data_prep/daymeans/refseries/qcfiles/"), pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
  if(length(qcf)>0){
    write_flags(file, qcfile=qcf,
                outpath=outpath)
  }  
  
  ## all files with qc need this
  if(inventory[varind[ff],"gap_filling_data"]!="no"){
    print("fill gaps")
    out <- read_sef(file, all = T)
    out[grepl("qc=",out$Meta),"Value"] <- NA
    #out <- fill_gaps(out,gap_data=inventory[varind[ff],"gap_filling_data"],xlon=xlon,xlat=xlat,var=var,id=id) 
    
    newout <- check_dates(out[,2:ncol(out)])
    newout <- cbind(rep("ta", nrow(newout)), newout)
    out <- fill_gaps(newout,gap_data=inventory[varind[ff],"gap_filling_data"],xlon=xlon,xlat=xlat,var=var,id=id) 
    write_sef(out[,c(2:8)], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), paste0("merged_",var), 
              xlat, xlon, inventory[varind[ff],"alt_hist"], sou=toupper(cllps(ref,"-")),
              link="various", 
              units="C", stat="mean", meta=out$Meta,
              period="day",  note="daily_merged", keep_na=TRUE)
  }
     
  rm(outl,out,xlat,xlon,var)
  } else{

  ## 1) if reference data is from mch
  if(ref=="mch"){
    refhelp <- unlist(strsplit(refstn[ff],","))
    if(substr(refhelp,5,7)=="ths"){
      dat <- read.table(paste0(refpath,refhelp), header = T, skip=2,na.strings = "-", sep=";")
      dat <- dat[dat$stn==gsub("_ta","",id),]
      rows <- ifelse(id%in%c("NEU_ta","LUZ_ta"),1,6)
      meta <- read.table(paste0(refpath,gsub("data","legend",refstn[ff])),skip=31,nrow=rows,
                         colClasses = rep("character",7),strip.white=T)
      meta <- meta[meta$V1==gsub("_ta","",id),]
      clm <- "ths200d0"
    } else{
      dat <- read.table(paste0(refpath,refhelp), header = T, skip=2,na.strings = "-")
      meta <- read.table(paste0(refpath,gsub("data","legend",refstn[ff])),skip=31,nrow=1,
                         colClasses = rep("character",7),strip.white=T)
      clm <- "tre200d0"
      }
    
    ## https://rdrr.io/github/falling-fruit/fruitr/man/ch1903_to_wgs84.html
    ch <- fruitr::ch1903_to_wgs84(as.integer(c(substr(meta["V6"],1,6),substr(meta["V6"],8,14))))
    xlon <- ch[1]
    xlat <- ch[2]
    
    out <- data.frame(Var=rep("ta", nrow(dat)),
                      Year=as.integer(substr(dat$time,1,4)),
                      Month=as.integer(substr(dat$time,5,6)),
                      Day=as.integer(substr(dat$time,7,8)),
                      Hour=rep(NA, nrow(dat)),
                      Minute=rep(NA, nrow(dat)),
                      Value=as.numeric(as.character(dat[,clm])),
                      Meta=rep("mchstn",nrow(dat)))
    out[,2:ncol(out)] <- check_dates(out[,2:ncol(out)])
    refind <- which(out$Year %in% c(refper[1]:refper[2]))
    out <- out[refind,]
    write_sef(out[,2:8], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), meta["V1"], 
              xlat, xlon, meta["V7"], "MCH", "idaweb", units="C", "mean", period="day", meta = out$Meta, note="daily", keep_na=TRUE)
    
    ## make a qc
    file <- list.files(outpath, pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
    qc(file, outpath=paste0("code/data_prep/daymeans/refseries/qcfiles/"))
    qcf <- list.files(paste0("code/data_prep/daymeans/refseries/qcfiles/"), pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
    if(length(qcf)>0){
      write_flags(file, qcfile=qcf,
                  outpath=outpath)
    }
    if(inventory[varind[ff],"gap_filling_data"]!="no"){
        print("fill gaps")
        out <- read_sef(file, all = T)
        out[grep("qc=",out$Meta),"Value"] <- NA
        newout <- check_dates(out[,2:ncol(out)])
        newout <- cbind(rep("ta", nrow(newout)), newout)
        out <- fill_gaps(newout,gap_data=inventory[varind[ff],"gap_filling_data"],xlon=xlon,xlat=xlat,var=var,id=id) 
        write_sef(out[,c("Year","Month","Day","Hour","Minute","Value")], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), meta["V1"], 
                  xlat, xlon, meta["V7"], "MCH", "idaweb", units="C", "mean", period="day", meta = out$Meta, note="daily", keep_na=TRUE)
        
      }
  }
  ## 2) if reference data is from eobs
  if(ref=="eob"){
    out <- read_eobs(path=get(paste0("eobs_",var)),xlon=xlon,xlat=xlat,var=var)
    out[,2:ncol(out)] <- check_dates(out[,2:ncol(out)])
    
    refind <- which(out$Year %in% c(refper[1]:refper[2]))
    out <- out[refind,]
    write_sef(out[,2:8], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), paste0("eobs_",var), 
              xlat, xlon, inventory[varind[ff],"alt_hist"], sou="EOBS",
              link="http://surfobs.climate.copernicus.eu/dataaccess/access_eobs.php", 
              units="C", stat="mean", meta=rep("closest_gp_eobs", length(refind)),
              period="day",  note="daily", keep_na=TRUE)
  }
  ## 3) if reference data is from ecad
  if(ref=="ECA"){
   
    dat <- read.table(paste0(refpath,"ecad/",refstn[ff]), header = T, skip=20, sep=",",na.strings=-9999)
    meta <- read.table(paste0(refpath,"ecad/",gsub("/.*","",refstn[ff]),"/stations.txt"),
                       skip=19,sep=",",colClasses = rep("character",6), strip.white=T, allowEscapes=T)
    meta <- meta[which(meta$V1==dat$STAID[1]),]
    lon  <- as.numeric(substr(meta["V5"],2,4))+as.numeric(substr(meta["V5"],6,7))/60+as.numeric(substr(meta["V5"],9,10))/3600
    lat  <- as.numeric(substr(meta["V4"],2,3))+as.numeric(substr(meta["V4"],5,6))/60+as.numeric(substr(meta["V4"],8,9))/3600
    realend <- which(sapply(1:nrow(dat),function(i) all(is.na(dat[i:nrow(dat),4]))))[1]
    dat <- dat[1:(realend-1),]
    
    out <- data.frame(Var=rep("ta", nrow(dat)),
                      Year=as.integer(substr(dat$DATE,1,4)),
                      Month=as.integer(substr(dat$DATE,5,6)),
                      Day=as.integer(substr(dat$DATE,7,8)),
                      Hour=rep(NA, nrow(dat)),
                      Minute=rep(NA, nrow(dat)),
                      Value=dat[,"TG"]*0.1,
                      Meta=rep("ECAD",nrow(dat)))
    out$Value[which(dat[,"Q_TG"] %in% c(1,9))] <- NA
    out[,2:ncol(out)] <- check_dates(out[,2:ncol(out)])
    refind <- which(out$Year %in% c(refper[1]:refper[2]))
    out <- out[refind,]
  
    write_sef(out[,2:8], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), meta["V2"], 
              lat=lat, lon=lon, meta["V6"], sou="ECAD", link="",
              units ="C", stat="mean", period="day", metaHead = "",meta=out$Meta, note="daily", keep_na=TRUE)
    
    ## make a qc
    file <- list.files(outpath, pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
    qc(file, outpath=paste0("code/data_prep/daymeans/refseries/qcfiles/"))
    qcf <- list.files(paste0("code/data_prep/daymeans/refseries/qcfiles/"), pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
    if(length(qcf)>0){
      write_flags(file, qcfile=qcf,
                  outpath=outpath)
     }
    if(inventory[varind[ff],"gap_filling_data"]!="no"){
        print("fill gaps")
        out <- read_sef(file, all = T)
        out[grepl("qc=",out$Meta),"Value"] <- NA
        newout <- check_dates(out[,2:ncol(out)])
        newout <- cbind(rep("ta", nrow(newout)), newout)
        out <- fill_gaps(newout,gap_data=inventory[varind[ff],"gap_filling_data"],xlon=xlon,xlat=xlat,var=var,id=id) 
        write_sef(out[,c(2:8)], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), meta["V2"], 
                  lat=lat, lon=lon, meta["V6"], sou="ECAD", link="",
                  units ="C", stat="mean", period="day", metaHead = "",meta=out$Meta, note="daily", keep_na=TRUE)
        
        }
      
    
    
    rm(out,dat,meta)
  }
  ## 4) if reference data is from ecad
  if(ref=="Wea"){
    refhelp <- unlist(strsplit(refstn[ff],","))
    dat <- read_sef(paste0("code/data_prep/daymeans/qced/",refhelp), all=T)
    meta <- read_meta(paste0("code/data_prep/daymeans/qced/",refhelp))
    out <- data.frame(Var=rep("ta", nrow(dat)),
                      Year=dat$Year,
                      Month=dat$Month,
                      Day=dat$Day,
                      Hour=dat$Hour,
                      Minute=dat$Minute,
                      Value=dat$Value,
                      Meta=dat$Meta)
    out[,2:ncol(out)] <- check_dates(out[,2:ncol(out)])
    refind <- which(out$Year %in% c(refper[1]:refper[2]))
    out <- out[refind,]
    
    write_sef(out[,2:8], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), meta["name"], 
              lat=meta["lat"], lon=meta["lon"], meta["alt"], sou="WeaR", link="",
              units ="C", stat="mean", period="day", metaHead = "",meta=out$Meta, note="daily", keep_na=TRUE)
    
    ## make a qc
    file <- list.files(outpath, pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
    qc(file, outpath=paste0("code/data_prep/daymeans/refseries/qcfiles/"))
    qcf <- list.files(paste0("code/data_prep/daymeans/refseries/qcfiles/"), pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
    if(length(qcf)>0){
      write_flags(file, qcfile=qcf,
                  outpath=outpath)
     } 
    if(inventory[varind[ff],"gap_filling_data"]!="no"){
        print("fill gaps")  
        out <- read_sef(file, all = T)
        out[grepl("qc=",out$Meta),"Value"] <- NA
        newout <- check_dates(out[,2:ncol(out)])
        newout <- cbind(rep("ta", nrow(newout)), newout)
        out <- fill_gaps(newout,gap_data=inventory[varind[ff],"gap_filling_data"],xlon=xlon,xlat=xlat,var=var,id=id) 
        write_sef(out[,c("Year","Month","Day","Hour","Minute","Value")], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), meta["name"], 
                  lat=meta["lat"], lon=meta["lon"], meta["alt"], sou="WeaR", link="",
                  units ="C", stat="mean", period="day", metaHead = "",meta=out$Meta, note="daily", keep_na=TRUE)
      }
  }
  ## 5) if reference from tabs      
  if(ref=="tab"){
        dat <- list()
        ncp <- nc_open(tabs)
        lons <- ncvar_get(ncp, "lon")
        lats <- ncvar_get(ncp, "lat")
        coords <- data.frame(lon=as.vector(lons), lat=as.vector(lats))
        
        ## get coordinates of station
        xlon <- as.numeric(inventory[varind[ff],"lon_hist"])
        xlat <- as.numeric(inventory[varind[ff],"lat_hist"])
        
        ## Select closest grid point on land (limit the search to a square of 25 grid points)       
        i_grid <- which.min(distGeo(c(xlon,xlat), coords))
        i <- ifelse(id == "ZUG_ta", 2, 1)
        i_lonlat <- which(lons == coords$lon[i_grid], arr.ind = T)[i,]
        #i_lat <- which(lats == coords$lat[i_grid], arr.ind = T)
        dat$dat <- ncvar_get(ncp, "TabsD", c(i_lonlat[1],i_lonlat[2],1), c(1,1,-1))
        dat$time <- nc_time_p(ncp,timename = "time")
        
        out <- data.frame(Year=lubridate::year(dat$time),
                                Month=lubridate::month(dat$time),
                                Day=lubridate::day(dat$time),
                                Hour=rep(NA, nrow(dat$dat)),
                                Minute=rep(NA, nrow(dat$dat)),
                                Value=round(dat$dat,2),
                                Meta=rep("TabsD",nrow(dat$dat)))
        out <- check_dates(out)
        refind <- which(out$Year %in% c(refper[1]:refper[2]))
        out <- out[refind,]
        
        write_sef(out[,1:7], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), inventory[varind[ff],"Name"], 
                  lat=xlat, lon=xlon, inventory[varind[ff],"alt_hist"], sou="TabsD", link="",
                  units ="C", stat="mean", period="day", metaHead = "",meta=out$Meta, note="daily", keep_na=TRUE)
        
      }
    }
  } 
  }


#### plot all reference series for temp
seffiles_ta <- list.files(path="code/data_prep/daymeans/refseries/sef/","_ta_daily", full.names = T)
seffiles_ta <- seffiles_ta[!grepl("KAR|qc_",seffiles_ta)] # Karlsruhe only for pressure correction not needed here
sef <- lapply(seffiles_ta, read_sef, all=T)
sef <- lapply(sef, function(x) { 
  if(any(grepl("qc=",x$Meta))){x$Value[grepl("qc=",x$Meta)] <- NA}
  return(x)
})
sefnames <- gsub(".*/","",seffiles_ta)
names(sef) <- sefnames

## Control plots
png(paste0("code/data_prep/plots/refseries/ta/all_series_ta_",Sys.Date(),".png"), width = 2000, height=4500, res=160)
par(mfrow=c(length(seffiles_ta),1), mar=c(2,2,1,1))
for(st in 1:length(seffiles_ta)){
  cols <- ifelse(grepl("pred.val",sef[[st]]$Meta),"forestgreen","black")
  plot(pdate(sef[[st]][,2:4]),sef[[st]][,"Value"], type="p",pch=19,cex=0.4, col=cols, xlim=as.Date(c("1961-01-01","2020-12-31")))
  mtext(side=3, sefnames[st])
}
dev.off()

sef <- lapply(sef, function(x){
  print("x")
  doys <- lubridate::yday(pdate(x[,2:4]))
  doymeans <- aggregate(as.numeric(x$Value), by=list(doys), "mean", na.rm=T)
  x$dailyanoms <- x$Value - doymeans$x[doys]
  return(x)  
})

png(paste0("code/data_prep/plots/refseries/ta/all_series_dailyanoms_ta_",Sys.Date(),".png"), width = 2000, height=4500, res=160)
par(mfrow=c(length(seffiles_ta),1), mar=c(2,2,1,1))
for(st in 1:length(seffiles_ta)){
  cols <- ifelse(grepl("pred.val",sef[[st]]$Meta),"forestgreen","black")
  plot(pdate(sef[[st]][,2:4]),sef[[st]][,"dailyanoms"], type="p",pch=19,cex=0.4, col=cols, xlim=as.Date(c("1961-01-01","2020-12-31")))
  mtext(side=3, sefnames[st])
}
dev.off()


rm(list=ls())
