########################################################
############## check for breakpoints in refseries
########################################################

rm(list=ls())

require(dataresqc)
require(geosphere)
require(ncdf4)
require(XLConnect)
require(lubridate)
require(qmap)
require(ClimIndVis)

source("code/data_prep/scripts/C000_helpfun.R")
source("code/data_prep/scripts/C005_helpfun_ref_new.R")
source("code/data_prep/RHtests-master/RHtests-master/V4_files/RHtestsV4_20190301.r")

refpath <- "code/data_prep/daymeans/refseries/"

##############################
#### create hom input files
##############################

seffiles <- list.files(paste0(refpath,"/sef"), full.names = T)
seffiles <- seffiles[!grepl("qc_|hom_|files_|RData", seffiles)]
seffiles <- seffiles[grepl("ta_ref",seffiles)]
stndat <- lapply(seffiles, function(s){return(read_sef(s,all=T))})
stnmeta <- sapply(seffiles, function(s){return(read_meta(s))})
stnmeta <- t(stnmeta)
id <- paste0(stnmeta[,"id"])
rownames(stnmeta) <- id
names(stndat) <- id
id_sef <- id

## set value with flag to NA
stndat <- lapply(stndat, function(s){
  s[grepl("qc=",s$Meta),"Value"]<- NA
  return(s)})

## create matrix of all data
yr <- range(sapply(stndat, function(x) range(x[,"Year"])))
daterange <- seq(as.Date(paste0(yr[1],"-01-01")),as.Date(paste0(yr[2],"-12-31")),by="day")
dattot <- matrix(NA, ncol=length(stndat),nrow=length(daterange))
colnames(dattot) <- as.character(id)
dindm <- list()

for(ff in 1:length(stndat)){
  y <- stndat[[ff]]
  dindm[[ff]] <- daterange %in% pdate(y[,c("Year","Month","Day")])
  dinds<-  pdate(y[,c("Year","Month","Day")]) %in% daterange
  dattot[dindm[[ff]],ff] <- y$Value[dinds]
}

climindvis <- make_object(tavg=t(dattot), dates_tavg = daterange,lon=stnmeta[,"lon"],lat=stnmeta[,"lat"], data_info = list(type="p",date_format="t1d",data_name="EIP"))
stnmons <- calc_index(climindvis, index="mean",var="tavg", aggt="monthly")

if (!"hom_output_ref" %in% list.dirs(refpath,full.names=FALSE)) {
  dir.create(path = paste0(refpath,"/hom_output_ref"))
  dir.create(path = paste0(refpath,"/hom_input_ref"))
}

for(ff in 1:length(stndat)){
  print(names(stndat)[ff])
  datf <- data.frame(year=rep(as.integer(stnmons$index_info$years),each=12),
                     mon= rep(c(1:12),length(stnmons$index_info$years)),
                     day=rep(00,length(rep(c(1:12),length(stnmons$index_info$year)))),
                     value= round(as.vector(stnmons$index[ff,,]),2))
  write.table(datf,file=paste0(refpath,"/hom_input_ref/",id[ff],".dat"),quote=F,row.names = F,col.names = F)
}
##############################
#### check for breakpoints
##############################
### use homogenized versions of mch as reference series
### for temperature

input <- list.files(path=paste0(refpath,"/hom_input_ref/"))
input <- input[grepl("ta_ref",input)]
input <- input[!grepl("KAR",input)]
id <- gsub(".dat","",input)
id_input <- c("BOL_ta_ref","BUS_ta_ref","HOH_ta_ref","MIL_ta_ref",
            "PAD_ta_ref","ROV_ta_ref","SHA_ta_ref","TOR_ta_ref")
id_refs <- c("LUG_ta_ref","BAS_ta_ref", "STG_ta_ref","LUG_ta_ref",
             "LUG_ta_ref","LUG_ta_ref","STG_ta_ref","LUG_ta_ref")

for(ff in 1:length(id_input)){
  if (!id_input[ff] %in% list.dirs(paste0(refpath,"/hom_output_ref/"),full.names=FALSE)) {
    dir.create(path = paste0(refpath,"/hom_output_ref/",id_input[ff],"/"))
  }
  FindU.wRef(Bseries=paste0(refpath,"/hom_input_ref/",id_input[ff],".dat"),MissingValueCode = "NA", p.lev=0.95,Iadj=10000,Mq=10,Ny4a=0,
             output = paste0(refpath,"/hom_output_ref/",id_input[ff],"/",id_input[ff]),
             Rseries=paste0(refpath,"/hom_input_ref/",id_refs[ff],".dat"))
}

### estimate adjustements
### !!!!!!!!! Then check all the adjustments, delete those, that are not significant re-call the function!!!!!!!!!

for (ff in 1:length(id_input)){
  print(id_input[ff])  
  StepSize.wRef(Bseries=paste0(refpath,"/hom_input_ref/",id_input[ff],".dat"),
                  Rseries=paste0(refpath,"/hom_input_ref/",id_refs[ff],".dat"),
                  InCs = paste0(refpath,"/hom_output_ref/",id_input[ff],"/",id_input[ff],"_mCs.txt"),
                  MissingValueCode = "NA", p.lev=0.95,Iadj=10000,Mq=10,Ny4a=0,output = paste0(refpath,"/hom_output_ref/",id_input[ff],"/",id_input[ff]))
}

### create final homogenized sef-files with adjustements estimated above 
for (ff in 1:(length(id_input))){
  print(id_input[ff])  
  
  # read sef
  f_ind <- which(id_sef %in% id_input[ff])
  sf <- read_sef(seffiles[f_ind],all=T)
  xo <- which(grepl("qc",sf$Meta))
  if(length(xo)>0) {sf[xo,"Value"] <-NA}
  meta <- read_meta(seffiles[f_ind])
  
  sefdata <- sf[,c("Year","Month","Day","Hour","Minute","Value","Meta")]
  # read hom vals
  Fdat <- read.table(paste0(refpath,"/hom_output_ref/",id_input[ff],"/",id_input[ff],"_F.dat"))
  qmt <- 5 #ifelse(all(is.na(Fdat[,11])),5,11) -> don't use the qm based estimates, because they are estimated based on the EKF reference
  
  homadj_monthly <- Fdat[,qmt] - Fdat[,3] 
  # create daily adjustments using a spline?
  mimo <- Fdat[,2]+15
  mohelp <- as.Date(paste0(substr(mimo,1,4),"-",substr(mimo,5,6),"-",substr(mimo,7,8)))
  lmo <- lubridate::days_in_month(as.Date(paste0(substr(Fdat[nrow(Fdat),2],1,4),"-",substr(Fdat[nrow(Fdat),2],5,6),"-01")))
  dhelp <- seq(as.Date(paste0(substr(Fdat[1,2],1,4),"-",substr(Fdat[1,2],5,6),"-01")),
               as.Date(paste0(substr(Fdat[nrow(Fdat),2],1,4),"-",substr(Fdat[nrow(Fdat),2],5,6),"-",lmo)),
               by="day")
  xout <- which(dhelp %in% pdate(sefdata[,1:3]))
  which(!pdate(sefdata[,1:3]) %in% dhelp[xout])
  
  ## don't use spline. because it makes weird step to next period...
  homadj_daily <- homadj_monthly[match(substr(dhelp[xout],1,7),substr(mohelp,1,7))]
  # add hom adjust
  sefdata[,"Value"] <- round(sefdata[,"Value"] + homadj_daily,2)
  pipe <- ifelse(sefdata[,"Meta"]=="","","|")
  sefdata[,"Meta"] <- paste0(sefdata[,"Meta"],pipe,"homadj=",round(homadj_daily,2))
  
  # check for complete series
  sefdata <- check_dates(sefdata)
  
  write_sef(Data = sefdata[, 1:7],
            outpath = "code/data_prep/daymeans/refseries/hom/",
            variable = "ta",
            cod = meta["id"],
            nam = meta["name"],
            lat = meta["lat"],
            lon = meta["lon"],
            alt = meta["alt"],
            link = meta["link"],
            sou = "WeaR",
            units = meta["units"],
            stat = "mean",
            metaHead = meta["meta"],
            meta=sefdata$Meta,
            period = "day",
            time_offset =0,
            keep_na = TRUE,
            note="hom")
}



