###############################################
############## Merge long series
##############################################

#### visualize all series
#### correlations
#### are there annual cycles of differences?
#### compare distributions
#### breakpoints

rm(list=ls())

require(maps)
require(ClimIndVis)
require(RColorBrewer)
require(dataresqc)
require(lubridate)

source('code/data_prep/scripts/C000_helpfun.R')
source("code/data_prep/scripts/C005_helpfun_ref.R")
source("code/data_prep/RHtests-master/RHtests-master/V4_files/RHtestsV4_20190301.r")

### path
homref <- "code/data_prep/daymeans/histseries/hom/"
finpath <- "code/data_prep/daymeans/histseries/merged/"
plotpath <- "code/data_prep/plots/genf/"

distrplot <- F

### data
seffiles <- list.files(homref, full.names=T, pattern="Geneve|GVE")
id <- sapply(seffiles, function(x){read_meta(x)["id"]})
nam <- sapply(seffiles, function(x){read_meta(x)["name"]})
varf <- sapply(seffiles, function(x){read_meta(x)["var"]})
lats <- sapply(seffiles, function(x){read_meta(x)["lat"]})
lons <- sapply(seffiles, function(x){read_meta(x)["lon"]})
alts <- sapply(seffiles, function(x){read_meta(x)["alt"]})
allsefs <- lapply(seffiles, function(x){
  sef <- read_sef(x, all=T)
  sef$Value[grepl("qc=",sef$Meta)] <- NA
  return(sef)
})
names(allsefs) <- paste0(id,"_",varf)

dates_range <- as.Date(c("1761-01-01","1863-12-31"))
alldates <- seq(dates_range[1], dates_range[2], by="day")

###############
### Genf
###############
pind <- which(!varf=="ta")

##### Comparison

yr <- range(sapply(allsefs[pind], function(x) range(x[,"Year"])))
daterange <- seq(as.Date(paste0(yr[1],"-01-01")),as.Date(paste0("1863-12-31")),by="day")
dattot <- data.frame(matrix(NA, ncol=length(pind)+4,nrow=length(daterange)))
colnames(dattot) <- c("year","mon","day",as.character(id[pind]),"Meta")
dindm <- list()

for(ff in 1:length(pind)){
  y <- allsefs[[pind[ff]]]
  dindm[[ff]] <- daterange %in% pdate(y[,c("Year","Month","Day")])
  dinds<-  pdate(y[,c("Year","Month","Day")]) %in% daterange
  dattot[dindm[[ff]],ff+3] <- y$Value[dinds]
}
dattot[,c("year","mon","day")] <- cbind(year(daterange),month(daterange),day(daterange))


if(distrplot){
  doys <- yday_without_29feb(daterange)
  dist_all_ts <- lapply(1:length(pind), function(y){
    datstn <- dattot[,y+3]
    dist <- t(sapply(1:365, function(x){quant_window_fun(dat=datstn,dates=daterange,doy=x)} ))
    return(dist)
  })
  names(dist_all_ts) <- colnames(dattot[4:5])
  
  png(paste0(plotpath,"distribution_GVE_Senebier_p_q50_q10_q90_",Sys.Date(),".png"), width = 2000, height = 1000, res=200)
  ll <-1.5
  par(mfrow=c(1,1), mar=c(2,2,1,2))
  plot(1:365,dist_all_ts$GE01_Geneve_Senebier[,3], type="l", ylim=c(940,990), col="turquoise1", lwd=ll)
  points(1:365,dist_all_ts$GVE[,3], type="l", col="darkred", lwd=ll)
  points(1:365,dist_all_ts$GE01_Geneve_Senebier[,5], type="l", lty=3, col="turquoise1", lwd=ll)
  points(1:365,dist_all_ts$GVE[,5], type="l", lty=3, col="darkred", lwd=ll)
  points(1:365,dist_all_ts$GE01_Geneve_Senebier[,1], type="l", lty=3, col="turquoise1", lwd=ll)
  points(1:365,dist_all_ts$GVE[,1], type="l", lty=3,col="darkred", lwd=ll)
  abline(h=seq(900,990,10),lty=3, col="gray65")
  legend("bottom", legend=c("Senebier median","Senebier q10/q90","GVE","GVE q10/q90"),  
         col=c("turquoise1","turquoise1","darkred","darkred"),lty = c(1,3,1,3,1,3),bty="n", cex=1.7)
  title("Jahresgang unterschiedl.Perioden")
  dev.off()
  
  
}


############################################################
##### just merge the serie and then check for inhomogenity

### ta
range(allsefs$GVE_ta$Year)
range(allsefs$GE01_Geneve_Senebier_ta$Year)
yrr <- c(1782,1863)
daterange <- seq(as.Date(paste0(yrr[1],"-01-01")),as.Date(paste0(yrr[2],"-12-31")), by="day")
sefdata <- data.frame(year=year(daterange),
                      mo=month(daterange),
                      d=day(daterange),
                      h=rep(NA, length(daterange)),
                      m=rep(NA,length(daterange)),
                      Value=rep(NA,length(daterange)),
                      Meta=rep(NA,length(daterange)))

gveind <- which(pdate(sefdata[,1:3]) %in% pdate(allsefs[["GVE_p"]][,c(2:4)]))
gesenind <- which(pdate(sefdata[,1:3]) %in% pdate(allsefs[["GE01_Geneve_Senebier_p"]][,c(2:4)]) )

sefdata[gveind,"Value"] <- allsefs[["GVE_p"]][,"Value"]
sefdata[gesenind,"Value"] <- allsefs[["GE01_Geneve_Senebier_p"]][,"Value"]
sefdata[gveind,"Meta"] <- "GVE_ta"
sefdata[gesenind,"Meta"] <-"GE01_Geneve_Senebier"
sefdata <- check_dates(sefdata)

####  calc monthly means for break detection
momeans <- aggregate(sefdata$Value, list(sefdata$year,sefdata$mo), "mean_Xpercna")
momeans <- momeans[order(paste0(momeans$Group.1,pad2(momeans$Group.2))),]

datf <- data.frame(year=momeans$Group.1,
                   mon= momeans$Group.2,
                   day=rep(00, nrow(momeans)),
                   value= round(momeans$x,2))
write.table(datf,file=paste0("code/data_prep/daymeans/histseries/merged/hom_input_merge/","WeaR_GVEmerged_p",".dat"),quote=F,row.names = F,col.names = F)
## comp series
ekf_p <- read.table("code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_p.dat")
dind <- paste0(datf$year,pad2(datf$mon)) %in% paste0(ekf_p$V1,pad2(ekf_p$V2))
eind <- paste0(ekf_p$V1,pad2(ekf_p$V2)) %in% paste0(datf$year,pad2(datf$mon))
diff <- data.frame(date=paste0(datf$year,pad2(datf$mon))[dind],diff=(datf$value[dind]- ekf_p$V4[eind]))

png(paste0(plotpath,"Diff_EKF400_zur_nothom_p_",Sys.Date(),".png"), width=2500)
plot(1:nrow(diff),diff$diff, type="l", xaxt="n")
title("Difference EKF400 - Merged GVE P (not hom)")
abline(v=paste0(datf$year,pad2(datf$mon))[dind][seq(1,12*140,12*5)], lty=3, col="gray65")
axis(side=1, at=seq(1, nrow(diff),12),momeans$Group.1[seq(1,nrow(diff),12)])
dev.off()
#########################################################################################################
## check for change points with ref sereis
#########################################################################################################

if (!"WeaR_GVEmerged_p" %in% list.dirs("code/data_prep/daymeans/histseries/merged/hom_output_merge/",full.names=FALSE)) {
  dir.create(path = "code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_GVEmerged_p/")}
FindU.wRef(Bseries="code/data_prep/daymeans/histseries/merged/hom_input_merge/WeaR_GVEmerged_p.dat",MissingValueCode = "NA", p.lev=0.95,Iadj=10000,Mq=10,Ny4a=0,
           output ="code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_GVEmerged_p/WeaR_GVEmerged_p",
           Rseries="code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_p.dat")

#########################################################################################################
## reestimate significant change points
#########################################################################################################

StepSize.wRef(Bseries="code/data_prep/daymeans/histseries/merged/hom_input_merge/WeaR_GVEmerged_p.dat",
              Rseries="code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_p.dat",
              InCs = "code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_GVEmerged_p/WeaR_GVEmerged_p_mCs.txt",
              MissingValueCode = "NA", p.lev=0.95,Iadj=10000,Mq=10,Ny4a=0,
              output = "code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_GVEmerged_p/WeaR_GVEmerged_p")

#########################################################################################################
## create final homogenized sef-files with adjustements estimated above 
#########################################################################################################

# read hom vals
Fdat <- read.table("code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_GVEmerged_p/WeaR_GVEmerged_p_F.dat")
qmt <- 5 #ifelse(all(is.na(Fdat[,11])),5,11) -> don't use the qm GVEed estimates, because they are estimated GVEed on the EKF reference
## set Fdat values to NA, if they are followed by a large number of NAs...

homadj_monthly <- Fdat[,qmt] - Fdat[,3] 
# expand monthly values to daily vals
mimo <- Fdat[,2]+15
mohelp <- as.Date(paste0(substr(mimo,1,4),"-",substr(mimo,5,6),"-",substr(mimo,7,8)))
lmo <- lubridate::days_in_month(as.Date(paste0(substr(Fdat[nrow(Fdat),2],1,4),"-",substr(Fdat[nrow(Fdat),2],5,6),"-01")))
dhelp <- seq(as.Date(paste0(substr(Fdat[1,2],1,4),"-",substr(Fdat[1,2],5,6),"-01")),
             as.Date(paste0(substr(Fdat[nrow(Fdat),2],1,4),"-",substr(Fdat[nrow(Fdat),2],5,6),"-",lmo)),
             by="day")
xout <- which(dhelp %in% pdate(sefdata[,1:3]))

## don't use spline. because it makes weird step to next period...
homadj_daily <- homadj_monthly[match(substr(dhelp[xout],1,7),substr(mohelp,1,7))]
# add hom adjust
sefdata[,"Value"] <- round(sefdata[,"Value"] + homadj_daily,2)
pipe <- ifelse(sefdata[,"Meta"]=="","","|")
sefdata[,"Meta"] <- paste0(sefdata[,"Meta"],pipe,"homadj=",round(homadj_daily,2))

## plot GVEel
momeans_test <- aggregate(sefdata$Value, list(sefdata$year,sefdata$mo), "mean_Xpercna", na=0.25)
momeans_test <- momeans_test[order(paste0(momeans_test$Group.1,pad2(momeans_test$Group.2))),]
datehelp <- as.Date(paste0(momeans_test$Group.1,"-",pad2(momeans_test$Group.2),"-","15"))
ekftest <- read.table("code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_p.dat")
ekfind <- which(paste0(ekftest$V1,"-",pad2(ekftest$V2)) %in% paste0(momeans_test$Group.1,"-",pad2(momeans_test$Group.2)))
png(paste0(plotpath,"Diff_EKF400_homGVEel_p_",Sys.Date(),".png"), width=2500)
plot(datehelp,momeans_test$x - ekftest$V4[ekfind], type="l")
title("Difference EKF400 - Merged Genf P")
abline(v=datehelp[seq(1,12*140,12*5)], lty=3, col="gray65")
dev.off()

### plot daily sefdata
stncols <- c("black","blue3")
ser <- sefdata
doys <- lubridate::yday(pdate(ser[,1:3]))
doymeans <- aggregate(ser$Value, list(doys),"mean", na.rm=T)
dayanoms <- ser$Value-doymeans$x[doys]
metahelp <- gsub("|homadj=.*|monthly_mean_too_high|","",ser$Meta)
metahelp <- gsub("\\|","", metahelp)
mu <- unique(metahelp)
mu <- mu[!grepl("NANA",mu)]
colsel <- stncols[match(metahelp,mu)]

png(paste0(plotpath,"histperiod_p_genf_final_",Sys.Date(),".png"), width=3000,res=180)
par(mar=c(3,2,2,10))
plot(1:nrow(ser),dayanoms, pch=20, col=colsel,xlab="", ylab="", xaxt="n",cex=0.1, ylim=c(-30,30))
axis(side=1, at=which(yday(daterange)==1)[seq(1,length(which(yday(daterange)==1)),5)],
     labels = substr(as.character(daterange[which(yday(daterange)==1)[seq(1,length(which(yday(daterange)==1)),5)]]),1,4))
mtext(side=3,line=-1.2,text="Genf P", cex=0.8)
abline(v=which(yday(daterange)==1)[seq(1,length(which(yday(daterange)==1)),5)], lty=3, col="gray55")
par(xpd=TRUE)
legend("right", inset=c(-0.12,0), legend=mu, col=stncols,pch=20, bty="n", cex=0.8)
par(xpd=F)
dev.off()


write_sef(Data = sefdata[, 1:7],
          outpath = finpath,
          variable = "p",
          cod = "GVE",
          nam = "Geneve",
          lat = lats[1],
          lon = lons[1],
          alt = alts[1],
          link = "",
          sou = "WeaR",
          units = "hPa",
          stat = "mean",
          metaHead = "homogenised merged serie from CHIMES|Digihom",
          meta=sefdata$Meta,
          period = "day",
          time_offset =0,
          keep_na = TRUE,
          note="hom")




