###############################################
############## Merge long series
##############################################

rm(list=ls())

require(maps)
require(ClimIndVis)
require(RColorBrewer)
require(dataresqc)
require(lubridate)

source('code/data_prep/scripts/C000_helpfun.R')
source("code/data_prep/scripts/C005_helpfun_ref.R")
source("code/data_prep/RHtests-master/RHtests-master/V4_files/RHtestsV4_20190301.r")

distrplot <- F

### path
homref <- "code/data_prep/daymeans/histseries/hom/"
finpath <- "code/data_prep/daymeans/histseries/merged/"
plotpath <- "code/data_prep/plots/zurich/"

### data
seffiles <- list.files(homref, full.names=T, pattern="_ZH0|Zurich|Winterthur|Kuesnacht")
id <- sapply(seffiles, function(x){read_meta(x)["id"]})
id_full <- gsub("_hom.tsv|_not_homogenized.tsv","",gsub(".*/","",seffiles))
nam <- sapply(seffiles, function(x){read_meta(x)["name"]})
varf <- sapply(seffiles, function(x){read_meta(x)["var"]})
lats <- sapply(seffiles, function(x){read_meta(x)["lat"]})
lons <- sapply(seffiles, function(x){read_meta(x)["lon"]})
alts <- sapply(seffiles, function(x){as.numeric(read_meta(x)["alt"])})
allsefs <- lapply(seffiles, function(x){
  sef <- read_sef(x, all=T)
  sef$Value[grepl("qc=",sef$Meta)] <- NA
  return(sef)
})
names(allsefs) <- id_full
names(alts)<- id_full

## for plot later
dates_range <- as.Date(c("1761-01-01","1863-12-31"))
alldates <- seq(dates_range[1], dates_range[2], by="day")
zurp <- pind <- which(varf == "p")

## read in the current climate bern p file
zurich_ref <- read_sef("code/data_prep/daymeans/refseries/hom/WeaR_SMA_p_ref_19610101-20200630_p_nothom.tsv")
## clim
zurich_yclim <- aggregate(zurich_ref$Value, list(yday(pdate(zurich_ref[,2:4]))), function(x){quantile(x, probs=c(0.02,0.25,0.5,0.75,0.98), na.rm=T)})

#### plot zuri ######
png(paste0(plotpath,"histperiod_ponly_zuri_hommerge_",Sys.Date(),".png"), height = 800, width = 2000, res=180)
par(mfrow=c(1,1), oma=c(2,1,1,1))
par(mar=c(0,2,1,15))
#### pressure
stncols <- c("black","blue3","cornflowerblue","turquoise1","forestgreen","green","yellowgreen","yellow","orange",
             "darkred","tomato","pink", "purple","magenta")
plot(NA, xlab="", ylab="", xaxt="n",xlim=c(dates_range), cex=0.1, ylim=c(-30,30))
for(ii in 1:length(zurp)){
  ser <- allsefs[[zurp[ii]]]
  doys <- lubridate::yday(pdate(ser[,2:4]))
  doymeans <- aggregate(ser$Value, list(doys),"mean", na.rm=T)
  dayanoms <- ser$Value-doymeans$x[doys]
  points(pdate(ser[,2:4]),dayanoms, pch=20, col=stncols[ii],cex=0.5)
}
mtext(side=3,line=-1.2,text="Zurich P", cex=0.8)
abline(v=alldates[yday(alldates)==1][seq(1,length(alldates[yday(alldates)==1]),10)], lty=3, col="gray55")
par(xpd=TRUE)
legend("right", inset=c(-0.25,0), legend=names(allsefs)[zurp], col=stncols,pch=20, bty="n", cex=0.7)
par(xpd=F)

axis(side=1, at=alldates[yday(alldates)==1][seq(1,length(alldates[yday(alldates)==1]),5)],
     labels = as.character(alldates[yday(alldates)==1][seq(1,length(alldates[yday(alldates)==1]),5)]))
dev.off()

## create matrix of all data
yr <- range(sapply(allsefs[pind], function(x) range(x[,"Year"])))
daterange <- seq(as.Date(paste0(yr[1],"-01-01")),as.Date(paste0("1863-12-31")),by="day")
dattot <- data.frame(matrix(NA, ncol=length(pind)+4,nrow=length(daterange)))
colnames(dattot) <- c("year","mon","day",as.character(id[pind]),"Meta")
dindm <- list()

for(ff in 1:length(pind)){
  y <- allsefs[[pind[ff]]]
  dindm[[ff]] <- daterange %in% pdate(y[,c("Year","Month","Day")])
  dinds<-  pdate(y[,c("Year","Month","Day")]) %in% daterange
  dattot[dindm[[ff]],ff+3] <- y$Value[dinds]
}
dattot[,c("year","mon","day")] <- cbind(year(daterange),month(daterange),day(daterange))

doymeans <- matrix(unlist(apply(dattot[,4:(ncol(dattot)-1)],2, function(x){aggregate(x,list(yday_without_29feb(daterange)), mean, na.rm=T)$x})), nrow=365,ncol=ncol(dattot)-4)
dattotanoms <-dattot[,4:(ncol(dattot)-1)] - doymeans[yday_without_29feb(daterange),]
corrmat2 <- round(cor(dattotanoms, use="pairwise.complete.obs"),2)

png(paste0(plotpath,"zurich_cormat_panoms.png"), width = 1200, height = 1200, res=180)
par(mar=c(12,12,2,2))
cols <- colorRampPalette(RColorBrewer::brewer.pal(n=11-1,"Reds"))(11-1)
fields::image.plot(1:ncol(corrmat2),1:ncol(corrmat2),corrmat2, col=cols, breaks=seq(0,1,0.1),ylab="",xlab="", xaxt="n", yaxt="n")
axis(side=1, las=2, labels = id[pind], at=seq(1:ncol(corrmat2)))
axis(side=2, las=2,labels = id[pind], at=seq(1:ncol(corrmat2)))
abline(v=1:ncol(corrmat2), lty=2, col="gray65", lwd=0.5)
abline(h=1:ncol(corrmat2), lty=2, col="gray65", lwd=0.5)
dev.off()


### plot for every year all data points #####
redblues <- colorRampPalette(c("darkgreen","forestgreen","yellowgreen","yellow","cornflowerblue","blue","purple","magenta"))
stncols <- redblues(length(pind))

for (yr in unique(dattot[,"year"])){
  yrind <- dattot[,"year"]==yr
  print(yr)
  stns <- which(apply(dattot[yrind,4:ncol(dattot)],2, function(x){!all(is.na(x))}))
  if(length(stns)>0){
    png(paste0(plotpath,"years_p/dailydata_zurich_yr",yr,"_",Sys.Date(),".png"), width =1000, height=500)
    dr <- range(dattot[yrind,stns+3], na.rm=T)
    plot(NA, xlim=range(pdate(dattot[yrind,c(1:3)])),col=stncols[stns[1]], pch=19, ylab="",xlab=yr, ylim=dr + c(-3,3), cex=0.7, xaxt="n")
    axis(side=1, at=pdate(dattot[yrind,c(1:3)])[which(substr(pdate(dattot[yrind,c(1:3)]),8,10)=="-01")], pdate(dattot[yrind,c(1:3)])[which(substr(pdate(dattot[yrind,c(1:3)]),8,10)=="-01")])
    ydays <- yday(pdate(dattot[yrind,c(1:3)]))
    polygon(c(pdate(dattot[yrind,c(1:3)]), rev(pdate(dattot[yrind,c(1:3)]))), 
            c(caTools::runmean(zurich_yclim$x[ydays,1], k=10), caTools::runmean(rev(zurich_yclim$x[ydays,5]),10)), col=t_col("gray85",50), border=NA)
    polygon(c(pdate(dattot[yrind,c(1:3)]), rev(pdate(dattot[yrind,c(1:3)]))), 
            c(caTools::runmean(zurich_yclim$x[ydays,2], k=10), caTools::runmean(rev(zurich_yclim$x[ydays,4]),10)), col=t_col("gray75",50), border=NA)
    
    #plot(pdate(dattot[yrind,c(1:3)]), dattot[yrind, stns[1]+3], col=stncols[stns[1]], pch=19, ylab="",xlab=yr, ylim=dr + c(-3,3), cex=0.7)
    #points(pdate(dattot[yrind,c(1:3)]), ifelse(is.na(dattot[yrind, stns[1]+3]),dr[2],NA), col=stncols[stns[1]], pch=8, cex=0.7)
    ## add climatology for 1960-2020
    
    if(length(stns)>0){
      for(ss in 1:length(stns)){
        points(pdate(dattot[yrind,c(1:3)]), dattot[yrind, stns[ss]+3], col=stncols[stns[ss]], pch=19, cex=0.7)
        points(pdate(dattot[yrind,c(1:3)]), ifelse(is.na(dattot[yrind, stns[ss]+3]),dr[2] + c(ss/2) ,NA), col=stncols[stns[ss]], pch=8, cex=0.7)
      }
    }
    legend("bottom",  col = stncols[stns], legend = id_full[pind][stns], pch=19, bty="n")
    dev.off()
  }
}

##### merge the serie and then check for inhomogenity #####
temp_uetli <- read_sef(paste0(homref,"/CHIMES_ZH04_Uetliberg_18410801-18631130_ta_hom.tsv"), all=T)
uetlidat <- convert_to_sealevel(out=allsefs[["CHIMES_ZH04_Uetliberg_18410801-18631130_p"]][,2:9], temp_dat=temp_uetli,
                                alt=alts["CHIMES_ZH04_Uetliberg_18410801-18631130_p"]-alts["CHIMES_ZH01_Zurich_Ulrich_18360101-18521231_p"])
### ta
range(allsefs$ZH01_Zurich_Ott_ta$Year)
range(allsefs$ZH01_Zurich_Bruegger_ta$Year)

### ta
range(allsefs$'CHIMES_ZH01_Zurich_Ott_17560713-17691006_p'$Year)
range(allsefs$'CHIMES_ZH04_Uetliberg_18410801-18631130_p'$Year)
yrr <- c(1756,1863)
daterange <- seq(as.Date(paste0(yrr[1],"-01-01")),as.Date(paste0(yrr[2],"-12-31")), by="day")
sefdata <- data.frame(year=year(daterange),
                      mo=month(daterange),
                      d=day(daterange),
                      h=rep(NA, length(daterange)),
                      m=rep(NA,length(daterange)),
                      Value=rep(NA,length(daterange)),
                      Meta=rep(NA,length(daterange)))

### add meyer
meyind <- which(daterange %in% pdate(allsefs[["CHIMES_ZH01_Zurich_Meyer_17590601-17650331_p"]][,c(2:4)]) )
sefdata[meyind,"Value"] <- allsefs[["CHIMES_ZH01_Zurich_Meyer_17590601-17650331_p"]][,"Value"]
sefdata[meyind,"Meta"] <- "CHIMES_ZH01_Zurich_Meyer_17590601-17650331_p"
### add hirzel
hirzind <- which(daterange %in% pdate(allsefs[["CHIMES_ZH01_Zurich_Hirzel_17610101-17691231_p"]][,c(2:4)]) & is.na(sefdata$Value))
hirzind2 <- which(pdate(allsefs[["CHIMES_ZH01_Zurich_Hirzel_17610101-17691231_p"]][,c(2:4)]) %in% daterange[hirzind])
sefdata[hirzind,"Value"] <- allsefs[["CHIMES_ZH01_Zurich_Hirzel_17610101-17691231_p"]][hirzind2,"Value"]
sefdata[hirzind,"Meta"] <- "CHIMES_ZH01_Zurich_Hirzel_17610101-17691231_p"
### add ott where both are NA
ottind <- which(daterange %in% pdate(allsefs[["CHIMES_ZH01_Zurich_Ott_17560713-17691006_p"]][,c(2:4)]) & is.na(sefdata$Value))
ottind2 <- which(pdate(allsefs[["CHIMES_ZH01_Zurich_Ott_17560713-17691006_p"]][,c(2:4)]) %in% daterange[ottind])
sefdata[ottind,"Value"] <- allsefs[["CHIMES_ZH01_Zurich_Ott_17560713-17691006_p"]][ottind2,"Value"]
sefdata[ottind,"Meta"] <- "CHIMES_ZH01_Zurich_Ott_17560713-17691006_p"
### add Hirzel 2
hirzscndind <- which(daterange %in% pdate(allsefs[["PALAEO-RA_GCOS_Zurich_Hirzel_17700101-17861231_p"]][,c(2:4)]) )
sefdata[hirzscndind,"Value"] <- allsefs[["PALAEO-RA_GCOS_Zurich_Hirzel_17700101-17861231_p"]][,"Value"]
sefdata[hirzscndind,"Meta"] <- "PALAEO-RA_GCOS_Zurich_Hirzel_17700101-17861231_p"
### add muralt where both are NA
murind <- which(daterange %in% pdate(allsefs[["PALAEO-RA_GCOS_Zurich_Muralt_17600101-17930413_p"]][,c(2:4)]) & is.na(sefdata$Value))
murind2 <- which(pdate(allsefs[["PALAEO-RA_GCOS_Zurich_Muralt_17600101-17930413_p"]][,c(2:4)]) %in% daterange[murind])
sefdata[murind,"Value"] <- allsefs[["PALAEO-RA_GCOS_Zurich_Muralt_17600101-17930413_p"]][murind2,"Value"]
sefdata[murind,"Meta"] <- "PALAEO-RA_GCOS_Zurich_Muralt_17600101-17930413_p"
### start from later years and go back
### add Horner 23 wheres all is NA
horind <- which(daterange %in% pdate(allsefs[["PALAEO-RA_GCOS_Zurich_Horner_18120610-18350818_p"]][,c(2:4)]) & is.na(sefdata$Value) )
horind2 <-  which(pdate(allsefs[["PALAEO-RA_GCOS_Zurich_Horner_18120610-18350818_p"]][,c(2:4)]) %in% daterange[horind])
sefdata[horind,"Value"] <- allsefs[["PALAEO-RA_GCOS_Zurich_Horner_18120610-18350818_p"]][horind2,"Value"]
sefdata[horind,"Meta"] <- "PALAEO-RA_GCOS_Zurich_Horner_18120610-18350818_p"
### add Weiss wheres all is NA
weiind <- which(daterange %in% pdate(allsefs[["PALAEO-RA_GCOS_Zurich_Weiss_18231201-18240630_p"]][,c(2:4)]) & is.na(sefdata$Value) )
weiind2 <-  which(pdate(allsefs[["PALAEO-RA_GCOS_Zurich_Weiss_18231201-18240630_p"]][,c(2:4)]) %in% daterange[weiind])
sefdata[weiind,"Value"] <- allsefs[["PALAEO-RA_GCOS_Zurich_Weiss_18231201-18240630_p"]][weiind2,"Value"]
sefdata[weiind,"Meta"] <-"PALAEO-RA_GCOS_Zurich_Weiss_18231201-18240630_p"
### add Feer wheres all is NA
feerind <- which(daterange %in% pdate(allsefs[["PALAEO-RA_GCOS_Zurich_Feer_18070101-18271231_p"]][,c(2:4)]) & is.na(sefdata$Value) )
feerind2 <-  which(pdate(allsefs[["PALAEO-RA_GCOS_Zurich_Feer_18070101-18271231_p"]][,c(2:4)]) %in% daterange[feerind])
sefdata[feerind,"Value"] <- allsefs[["PALAEO-RA_GCOS_Zurich_Feer_18070101-18271231_p"]][feerind2,"Value"]
sefdata[feerind,"Meta"] <- "PALAEO-RA_GCOS_Zurich_Feer_18070101-18271231_p"
### add Ulrich
ulrind <- which(daterange %in% pdate(allsefs[["CHIMES_ZH01_Zurich_Ulrich_18360101-18521231_p"]][,c(2:4)]) & is.na(sefdata$Value) )
sefdata[ulrind,"Value"] <- allsefs[["CHIMES_ZH01_Zurich_Ulrich_18360101-18521231_p"]][,"Value"]
sefdata[ulrind,"Meta"] <-"CHIMES_ZH01_Zurich_Ulrich_18360101-18521231_p"
### add HOF wheres all is NA
hofind <- which(daterange %in% pdate(allsefs[["CHIMES_ZH01_Zurich_Hofmeister_18350101-18371231_p"]][,c(2:4)]) & is.na(sefdata$Value) )
hofind2 <-  which(pdate(allsefs[["CHIMES_ZH01_Zurich_Hofmeister_18350101-18371231_p"]][,c(2:4)]) %in% daterange[hofind])
sefdata[hofind,"Value"] <- allsefs[["CHIMES_ZH01_Zurich_Hofmeister_18350101-18371231_p"]][hofind2,"Value"]
sefdata[hofind,"Meta"] <- "CHIMES_ZH01_Zurich_Hofmeister_18350101-18371231_p"

### add Uetliberg reduced to Ulrich
uetind <- which(daterange %in% pdate(uetlidat[,c(1:3)]))
uetind2 <-  which(pdate(uetlidat[,c(1:3)]) %in% daterange[uetind])
sefdata[uetind,"Value"] <- uetlidat[uetind2,"Value"]
sefdata[uetind,"Meta"] <- "CHIMES_ZH04_Uetliberg_18410801-18631130_ta"


####  calc monthly means for break detection
momeans <- aggregate(sefdata$Value, list(sefdata$year,sefdata$mo), "mean_Xpercna")
momeans <- momeans[order(paste0(momeans$Group.1,pad2(momeans$Group.2))),]

datf <- data.frame(year=momeans$Group.1,
                   mon= momeans$Group.2,
                   day=rep(00, nrow(momeans)),
                   value= round(momeans$x,2))
write.table(datf,file=paste0("code/data_prep/daymeans/histseries/merged/hom_input_merge/","WeaR_ZURmerged_p",".dat"),quote=F,row.names = F,col.names = F)
## comp series
ekf_p <- read.table("code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_p.dat")
dind <- paste0(datf$year,pad2(datf$mon)) %in% paste0(ekf_p$V1,pad2(ekf_p$V2))
eind <- paste0(ekf_p$V1,pad2(ekf_p$V2)) %in% paste0(datf$year,pad2(datf$mon))
diff <- data.frame(date=paste0(datf$year,pad2(datf$mon))[dind],diff=(datf$value[dind]- ekf_p$V4[eind]))

png(paste0(plotpath,"Diff_EKF400_zur_nothom_p_",Sys.Date(),".png"), width=2500)
plot(1:nrow(diff),diff$diff, type="l")
title("Difference EKF400 - Merged Zurich P (not hom)")
abline(v=paste0(datf$year,pad2(datf$mon))[dind][seq(1,12*140,12*5)], lty=3, col="gray65")
dev.off()

#########################################################################################################
## check for change points with ref sereis
#########################################################################################################

if (!"WeaR_ZURmerged_p" %in% list.dirs("code/data_prep/daymeans/histseries/merged/hom_output_merge/",full.names=FALSE)) {
  dir.create(path = "code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_ZURmerged_p/")}
FindU.wRef(Bseries="code/data_prep/daymeans/histseries/merged/hom_input_merge/WeaR_ZURmerged_p.dat",MissingValueCode = "NA", p.lev=0.95,Iadj=10000,Mq=10,Ny4a=0,
           output ="code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_ZURmerged_p/WeaR_ZURmerged_p",
           Rseries="code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_p.dat")

#########################################################################################################
## reestimate significant change points
#########################################################################################################

StepSize.wRef(Bseries="code/data_prep/daymeans/homhistseries/merged/hom_input_merge/WeaR_ZURmerged_p.dat",
              Rseries="code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_p.dat",
              InCs = "code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_ZURmerged_p/WeaR_ZURmerged_p_mCs.txt",
              MissingValueCode = "NA", p.lev=0.95,Iadj=10000,Mq=10,Ny4a=0,
              output = "code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_ZURmerged_p/WeaR_ZURmerged_p")

#########################################################################################################
## create final homogenized sef-files with adjustements estimated above 
#########################################################################################################

# read hom vals
Fdat <- read.table("code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_ZURmerged_p/WeaR_ZURmerged_p_F.dat")
qmt <- 5 #ifelse(all(is.na(Fdat[,11])),5,11) -> don't use the qm ZURed estimates, because they are estimated based on the EKF reference
## set Fdat values to NA, if they are followed by a large numZUR of NAs...

homadj_monthly <- Fdat[,qmt] - Fdat[,3] 
# expand monthly values to daily vals
mimo <- Fdat[,2]+15
mohelp <- as.Date(paste0(substr(mimo,1,4),"-",substr(mimo,5,6),"-",substr(mimo,7,8)))
lmo <- lubridate::days_in_month(as.Date(paste0(substr(Fdat[nrow(Fdat),2],1,4),"-",substr(Fdat[nrow(Fdat),2],5,6),"-01")))
dhelp <- seq(as.Date(paste0(substr(Fdat[1,2],1,4),"-",substr(Fdat[1,2],5,6),"-01")),
             as.Date(paste0(substr(Fdat[nrow(Fdat),2],1,4),"-",substr(Fdat[nrow(Fdat),2],5,6),"-",lmo)),
             by="day")
xout <- which(dhelp %in% pdate(sefdata[,1:3]))

## don't use spline. because it makes weird step to next period...
homadj_daily <- homadj_monthly[match(substr(dhelp[xout],1,7),substr(mohelp,1,7))]
# add hom adjust
sefdata[,"Value"] <- round(sefdata[,"Value"] + homadj_daily,2)
pipe <- ifelse(sefdata[,"Meta"]=="","","|")
sefdata[,"Meta"] <- paste0(sefdata[,"Meta"],pipe,"homadj=",round(homadj_daily,2))

## plot basel
momeans_test <- aggregate(sefdata$Value, list(sefdata$year,sefdata$mo), "mean_Xpercna", na=0.25)
momeans_test <- momeans_test[order(paste0(momeans_test$Group.1,pad2(momeans_test$Group.2))),]
datehelp <- as.Date(paste0(momeans_test$Group.1,"-",pad2(momeans_test$Group.2),"-","15"))
ekftest <- read.table("code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_p.dat")
ekfind <- which(paste0(ekftest$V1,"-",pad2(ekftest$V2)) %in% paste0(momeans_test$Group.1,"-",pad2(momeans_test$Group.2)))

png(paste0(plotpath,"Diff_EKF400_homZURICH_p_",Sys.Date(),".png"), width=2500)
plot(datehelp,momeans_test$x - ekftest$V4[ekfind], type="l", xlab="C")
title("Difference EKF400 - Merged ZURn P")
abline(v=datehelp[seq(1,12*140,12*5)], lty=3, col="gray65")
dev.off()


### plot daily sefdata
#### temperature
stncols <- c("black","blue3","cornflowerblue","turquoise1","forestgreen","green","yellowgreen","yellow","orange",
             "darkred","tomato","pink", "purple","magenta")
ser <- sefdata
doys <- lubridate::yday(pdate(ser[,1:3]))
doymeans <- aggregate(ser$Value, list(doys),"mean", na.rm=T)
dayanoms <- ser$Value-doymeans$x[doys]
metahelp <- gsub("|homadj=.*|monthly_mean_too_high|","",ser$Meta)
metahelp <- gsub("\\|","", metahelp)
mu <- unique(metahelp)
mu <- mu[!grepl("NANA",mu)]
colsel <- stncols[match(metahelp,mu)]

png(paste0(plotpath,"histperiod_p_zuri_final_",Sys.Date(),".png"), width=3000,res=180)
par(mar=c(3,2,2,10))
plot(1:nrow(ser),dayanoms, pch=20, col=colsel,xlab="", ylab="", xaxt="n",cex=0.1, ylim=c(-30,30))
axis(side=1, at=which(yday(daterange)==1)[seq(1,length(which(yday(daterange)==1)),5)],
     labels = substr(as.character(daterange[which(yday(daterange)==1)[seq(1,length(which(yday(daterange)==1)),5)]]),1,4))
mtext(side=3,line=-1.2,text="Zurich P", cex=0.8)
abline(v=which(yday(daterange)==1)[seq(1,length(which(yday(daterange)==1)),5)], lty=3, col="gray55")
par(xpd=TRUE)
legend("right", inset=c(-0.12,0), legend=mu, col=stncols,pch=20, bty="n", cex=0.8)
par(xpd=F)
dev.off()
# 

write_sef(Data = sefdata[, 1:7],
          outpath = finpath,
          variable = "p",
          cod = "SMA",
          nam = "Zurich",
          lat = lats[1],
          lon = lons[1],
          alt = alts[1],
          link = "",
          sou = "WeaR",
          units = "hPa",
          stat = "mean",
          metaHead = "homogenised merged serie from CHIMES|PALAEO-RA",
          meta=sefdata$Meta,
          period = "day",
          time_offset =0,
          keep_na = TRUE,
          note="hom")





