##############################################################
############## Homogenization hist stations to reference
#############################################################

rm(list=ls())

require(maps)
require(ClimIndVis)
require(RColorBrewer)
require(dataresqc)
require(lubridate)
require(PCICt)

source("code/data_prep/scripts/Cget_ncdf.R")
source('code/data_prep/scripts/C000_helpfun.R')
source("code/data_prep/scripts/C005_helpfun_ref_new.R")

##########################
######## ekf400 ##########
##########################
load <- T
if(load){
  ekf <- load_object("code/data_prep/daymeans/histseries/homref/ekf_ref.RData")
  ekf_p <- ekf$ekf_p
  ekf_ta <- ekf$ekf_ta
  rm(ekf)
  } else{
    
    ekffile <- "data/ekf400/EKF400_ensmean_1730-2003.nc"
    cutx=c(-20,20)
    cuty=c(30,55)
    
    ### temperature
    ekf_ta <- list()
    ekf_ta <-   get_ncdf(ekffile,"air_temperature", sub=360, cutx=cutx,cuty=cuty)[c("lon","lat","dat")]
    ekf_ta$time <- as.PCICt(ncvar_get(nc_open(ekffile),"time")*86400, cal="360_day", origin="1600-01-01")
    
    ### pressure
    ekf_p <- list()
    ekf_p <-   get_ncdf(ekffile,"air_pressure_at_sea_level", sub=360, cutx=cutx,cuty=cuty)[c("lon","lat","dat")]
    ekf_p$time <-   as.PCICt(ncvar_get(nc_open(ekffile),"time")*86400, cal="360_day", origin="1600-01-01")
    ekf_p$dat <- ekf_p$dat/100
    
    ekf <- list(ekf_p=ekf_p,ekf_ta=ekf_ta)
    save(ekf, file="code/data_prep/daymeans/histseries/homref/ekf_ref.RData")
}

##########################
######## obs ##########
##########################
homorig <- "code/data_prep/daymeans/histseries/merged/"
refseries <- "code/data_prep/daymeans/refseries/hom/" 
homref <- "code/data_prep/daymeans/histseries/homref/"

### ref data
reffiles <- list.files(refseries, full.names=T, pattern="_ta|p_nothom")
rfnam <- as.vector(sapply(reffiles, function(x){read_meta(x)["id"]}))
sefref <- lapply(reffiles, function(x){read_sef(x, all=T)})
names(sefref) <- rfnam

### hist data
seffiles <- list.files(homorig, full.names=T,pattern="_ta|_p")
sfnam <- paste0(sapply(seffiles, function(x){read_meta(x)["id"]}),"_",sapply(seffiles, function(x){read_meta(x)["var"]}))
sefhist <- lapply(seffiles, function(x){read_sef(x, all=T)})
metahist <- lapply(seffiles, function(x){read_meta(x)})
names(sefhist) <- sfnam
names(metahist) <- sfnam

all(substr(sfnam,1,3) == substr(rfnam, 1,3))

### calc monthly means of ref and hist and ekf
momeanref <- matrix(unlist(lapply(sefref, function(x){aggregate(x$Value, list(x$Month), FUN=mean_Xpercna)$x})), nrow=12, byrow = F)
colnames(momeanref) <- names(sefref)
sh <- 8

for (sh in 1:length(sefhist)){
  if(any(grepl(sfnam[sh], rfnam))){
    # select closest grid in ekf
    ids <- names(sefhist)[sh]
    var <- metahist[[sh]][c("var")]
    ekf <- get(paste0("ekf_",var))
    lls <- as.numeric(metahist[[sh]][c("lon","lat")])
    lon <- which.min(abs(ekf$lon-lls[1]))
    lat <- which.min(abs(ekf$lat-lls[2]))
    ekf$time <- as.Date(as.character(ekf$time))
    ekf_ts <- data.frame(year=year(ekf$time),month=month(ekf$time), dat=ekf$dat[lon,lat,])
    
    refpind <- which(ekf_ts$year >= 1961)
    momeanekf_ref <- aggregate(ekf_ts[refpind,3],by=list(ekf_ts[refpind,"month"]),FUN="mean", na.rm=T)
    diff_ref <- momeanref[,which(grepl(ids,colnames(momeanref)))] - momeanekf_ref$x
    
    xind <- sefhist[[sh]]$Year >= 1763
    x <- sefhist[[sh]][xind,2:9]
    meta <- metahist[[sh]]
    ryrs <- range(x$Year[!is.na(x$Value)])
    nayrind <- which(x$Year %in% ryrs[1]:ryrs[2])
    momeanhist <- aggregate(x$Value[nayrind], list(x$Month[nayrind]), FUN=mean_Xpercna, na=0.9)$x
    
    hind <- which(ekf_ts$year %in% ryrs[1]:ryrs[2])
    momeanekf_hist <- aggregate(ekf_ts$dat[hind], list(ekf_ts$month[hind]), FUN=mean_Xpercna)
    diff_hist <- momeanhist - momeanekf_hist$x
    mon_diff <- round(diff_ref - diff_hist,2)
    xnew <- x
    xnew$Value <- x$Value + mon_diff[x$Month]
    
    write_sef(Data = xnew[, c(1:5,7)],
              outpath = homref,
              variable = var,
              cod = meta["id"],
              nam = meta["name"],
              lat = meta["lat"],
              lon = meta["lon"],
              alt = meta["alt"],
              link = meta["link"],
              sou = meta["source"],
              units = meta["units"],
              stat = "mean",
              metaHead = meta["meta"],
              meta=paste(xnew$Meta,"|homREFadj=",mon_diff[x$Month]),
              period = "day",
              time_offset =0,
              keep_na = TRUE,
              note="homref")
  }

}





