
#require(ncdf4)

get_nc_names <- function(ruta){
  
  file_nc <- ncdf4::nc_open(ruta)
  print(names(file_nc$var))
  ncdf4::nc_close(file_nc)
  return(names(file_nc$var))
}

get_levs <- function(ruta,levname="lev"){
  
  file_nc <- ncdf4::nc_open(ruta)
  print(dimnames <- names(file_nc$dim))
  lev <- dimnames[grep(levname,dimnames)]
  lev <- file_nc$dim[[lev]]$vals
  ncdf4::nc_close(file_nc)
  return(lev)
}

get_ncdf <- function(ruta,varname,cutx=NULL,cuty=NULL,sub=NULL,yrange=NULL, spec=F,NAval=NULL, only=NULL, lev=NULL, newcoords=NULL){
  
  file_nc <- ncdf4::nc_open(ruta)
  dimnames <- names(file_nc$dim)
  if(length(newcoords)>0){
    lon <- dimnames[grep(newcoords[1],dimnames)]
    lat <- dimnames[grep(newcoords[2],dimnames)]
  } else{
    lon <- dimnames[grep("lo",dimnames)]
    lat <- dimnames[grep("la",dimnames)]
  }
  
  if(length(only)>0){
  
    if(only=="lat"){
       file_lat <- ncdf4::ncvar_get(file_nc, lat)
      if(length(file_lat)>1){if((file_lat[2]-file_lat[1])<0){file_lat <- rev(file_lat)}}
      if(!is.null(cuty)){file_lat <- file_lat[file_lat >= cuty[1] & file_lat <= cuty[2]]}
      return(list(lat=file_lat))
      }  
     if(only=="lon"){
       file_lon <- ncdf4::ncvar_get(file_nc, lon)
      
       if(!is.null(sub)) {
       if(sub==360){
        lon_trfd <- c(file_lon[which(file_lon<=180)],file_lon[which(file_lon>180)]-360)
        file_lon=switch((sub==360)+1,file_lon-sub,lon_trfd[order(lon_trfd)])
      } else {
        file_lon= c(file_lon[which(file_lon>180)]-360, file_lon[which(file_lon <= 180)])
      }}
      if(!is.null(cutx)){file_lon <- file_lon[file_lon >= cutx[1] & file_lon <= cutx[2]]}
      return(list(lon=file_lon))
      }  
  } else {
  
  file_dat <- ncdf4::ncvar_get(file_nc, varname)
    
  if(length(lev)>0){file_dat <- file_dat[,,lev,]} 
  dims <- dim(file_dat)
  
  if(any(grepl("tim|REFERENCE_TS",dimnames))){
    tim <- dimnames[grep("tim|REFERENCE_TS",dimnames)]
    
    ti <- file_nc$dim[[tim]]$units
    tinfo <- ifelse(grepl("hour", ti),"hours",ifelse(grepl("days since", ti),"days","unknown"))
    orig <- stringr::str_split(ti,"since ")[[1]][2]
    print(tinfo)
    twodim <- FALSE
    
      if(spec){
      t <- as.character(ncdf4::ncvar_get(file_nc, "time"))
      if(any(grepl(pattern=".5",t))){
        hour <- substr(t,10,11)
        steps <- length(unique(hour))
        hours <- rep(seq(0,24, by =24/steps)[1:steps],length.out=length(t))
        time  <- as.POSIXct(paste0(substr(t,1,4),"-",substr(t,5,6),"-",substr(t,7,8)," ", ClimIndVis:::pad2(hours,2),":00"))
      } else {
        time <- as.Date(paste0(substr(t,1,4),"-",substr(t,5,6),"-",substr(t,7,8)))
      }
      
    } else{
      time <- nc_time_p(file_nc,timename = tim) 
    }
    
  } else{twodim<-TRUE}
  
  if(!is.null(yrange)){
    yr <- lubridate::year(time)
    yind <- which(yr>=yrange[1] & yr <=yrange[2])
    file_dat <- ClimIndVis:::index_array(file_dat, dim=length(dims),value=yind)
    time <- time[yind]
    dims <- dim(file_dat)
  }
  
  if(length(lon)>0){
    
    file_lon <- ncdf4::ncvar_get(file_nc, lon)
    if(!is.null(sub)) {
      if(sub==360){
        lon_trfd <- c(file_lon[which(file_lon<=180)],file_lon[which(file_lon>180)]-360)
        file_lon=switch((sub==360)+1,file_lon-sub,lon_trfd[order(lon_trfd)])
        file_dat= file_dat[order(lon_trfd),,]
      } else {
        file_lon= c(file_lon[which(file_lon>180)]-360, file_lon[which(file_lon <= 180)])
      }
    }
    dims <- dim(file_dat)
    file_lat <- ncdf4::ncvar_get(file_nc, lat)
    if(length(file_lat)>1){
      if((file_lat[2]-file_lat[1])<0){
        file_lat <- rev(file_lat)
        
        values = lapply(dims, function(dd) 1:dd)
        values[[2]] <- rev(values[[2]])
        file_dat <- ClimIndVis:::index_array(x=file_dat,dim=1:length(dims), value=values)
      }
    }
    dims <- dim(file_dat)
    
    
  } else {
     
    file_lon <- NA
    file_lat <- NA
   }
  if(length(NAval)>0){
    file_dat[which(file_dat==NAval)] <- NA
  }
  
  if(!is.null(cutx) & !is.null(cuty)){
    
    dim3 <- switch((length(dims)>2)+1,NULL,1:dims[3])
    dim4 <- switch((length(dims)>3)+1,NULL,1:dims[4])
    ll <- list(which(file_lon >= cutx[1]& file_lon <= cutx[2]), 
               which(file_lat >= cuty[1] & file_lat <= cuty[2]),
               dim3,dim4)
    file_cut <- ClimIndVis:::index_array(x=file_dat, dim=1:length(dims),value=ll)
    file_lon.cut <- file_lon[file_lon >= cutx[1]& file_lon <= cutx[2]]
    file_lat.cut <- file_lat[file_lat >= cuty[1] & file_lat <= cuty[2]]
    file_list <- list(dat = file_cut, lon= file_lon.cut, lat=file_lat.cut, time=time)
  } else {
    file_list <- switch(twodim+1,list(dat = file_dat, lon= file_lon, lat=file_lat, time=time),list(dat = file_dat, lon= file_lon, lat=file_lat))
    
  }
  nc_close(file_nc)
  return(file_list)}
}



get_ncdf_lev <- function(ruta,varname,cutx=NULL,cuty=NULL,lev=NULL,sub=FALSE,yrange=NULL,tm=1){
  
  file_nc <- ncdf4::nc_open(ruta)
  file_dat <- ncdf4::ncvar_get(file_nc, varname)
  dimnames <- names(file_nc$dim)
  lon <- dimnames[grep("lo",dimnames)]
  lat <- dimnames[grep("la",dimnames)]
  ln <- ifelse(length(grep("lev",dimnames))>0,grep("lev",dimnames),grep("lv",dimnames))
  levname <- dimnames[ln]
  
  if(any(grepl("tim",dimnames))){
    tim <- dimnames[grep("tim",dimnames)]
    
    ti <- file_nc$dim[[tim]]$units
    tinfo <- ifelse(grepl("hour", ti),"hours",ifelse(grepl("day since", ti),"days","unknown"))
    orig <- stringr::str_split(ti,"since ")[[1]][2]
    print(tinfo)
    twodim <- FALSE
    
    if(spec){
      t <- as.character(ncdf4::ncvar_get(file_nc, "time"))
      if(any(grepl(pattern=".5",t))){
        hour <- substr(t,10,11)
        steps <- length(unique(hour))
        hours <- rep(seq(0,24, by =24/steps)[1:steps],length.out=length(t))
        time  <- as.POSIXct(paste0(substr(t,1,4),"-",substr(t,5,6),"-",substr(t,7,8)," ", ClimIndVis:::pad2(hours,2),":00"))
      } else {
        time <- as.Date(paste0(substr(t,1,4),"-",substr(t,5,6),"-",substr(t,7,8)))
      }
      
    } else{
      time <- nc_time_p(file_nc) 
    }
    
  } else{twodim<-TRUE}
  
  ddims <- as.character(length(dim(file_dat)))
  
  if(!is.null(yrange)){
    time2 <- as.Date(as.character(time), format=c("%Y%m%d"),origin="1900-01-01")
    file_dat <- file_dat[,,yind]
    
  }
  
  
  file_lon <- ncdf4::ncvar_get(file_nc, lon)
  if(!is.null(sub)) {
    file_lon=switch((sub==360)+1,file_lon-sub, c(file_lon[which(file_lon<=180)],file_lon[which(file_lon>180)]-360)[order(c(file_lon[which(file_lon<=180)],file_lon[which(file_lon>180)]-360))])
  }
  file_lat <- ncdf4::ncvar_get(file_nc, lat)
  if((file_lat[2]-file_lat[1])<0){
    file_lat <- rev(file_lat)
    file_dat <- switch(ddims,"4"=file_dat[,length(file_lat):1,,],"3"=file_dat[,length(file_lat):1,],"2"=file_dat[,length(file_lat):1])
  }
  
  if(!is.null(lev)){
    file_dat <- file_dat[,,lev,]
  } else {
    lev <- ncdf4::ncvar_get(file_nc, levname)
    
  }
  if(!is.null(cutx) & !is.null(cuty)){
    file_cut <- file_dat[file_lon >= cutx[1]& file_lon <= cutx[2], file_lat >= cuty[1] & file_lat <= cuty[2],,]
    file_lon.cut <- file_lon[file_lon >= cutx[1]& file_lon <= cutx[2]]
    file_lat.cut <- file_lat[file_lat >= cuty[1] & file_lat <= cuty[2]]
    #file_cut <- file_cut[,length(file_lat.cut):1]
    file_list <- list(dat = file_cut, lon= file_lon.cut, lat=file_lat.cut, time=time)
  } else {
    
    file_list <- switch((ddims=="2")+1,list(dat = file_dat, lon= file_lon, lat=file_lat, time=time, lev=lev),
                        list(dat = file_dat, lon= file_lon, lat=file_lat,lev=lev))
    
  }
  
  
  nc_close(file_nc)
  return(file_list)
}


nc_time_p <- function(nc,refdat=NA,timename="z|tim"){
  t.i <- grep(timename, names(nc$dim))
  t.units <- nc$dim[[t.i]]$units
  t.mul <- c(second=1,
             minute=60,
             hour=3600,
             day=24*3600,
             month=1,
             year=1) ## special cases for days-years
  t.vals <- nc$dim[[t.i]]$val
  mul.i <- pmatch(substr(t.units,1,3), names(t.mul))
  refdate <- as.POSIXct(gsub('.*since ', '', t.units), tz='UTC')
  if(!is.na(refdat)){
    return(refdate)
  } else{
    if (length(mul.i) == 1 & is.numeric(t.mul)){
      if (mul.i <= 4){
        t.out <- refdate + t.vals * t.mul[mul.i]
      } else {
        if (any(t.vals[!is.na(t.vals)] %% 1 != 0)){
          stop("Non-integer months or years are not supported")
        }
        t.out <- rep(refdate, length(t.vals))
        if (mul.i == 5) month(t.out) <- month(t.out) + t.vals
        if (mul.i == 6) month(t.out) <- month(t.out) + t.vals
        ## fix end of month problem
        for (dd in 1:3){
          if (any(is.na(t.out))){
            newref <- refdate
            day(newref) <- day(refdate) - dd
            ina <- is.na(t.out)
            t.out[ina] <- newref
            if (mul.i == 5) month(t.out[ina]) <- month(t.out[ina]) + t.vals[ina]
            if (mul.i == 6) year(t.out[ina]) <- year(t.out[ina]) + t.vals[ina]
          }
        }
      }
    } else {
      stop('Time coordinate format not implemented yet')
    }
    return(t.out)
    
  }
}
