# analogs_ncdf_extract.py
# create netCDF from best analogue dates
# 21.06.2019 adpated 2022-03-11
# -----------------------------------------------

# import packages
import numpy as np
import pandas as pd
import xarray as xr
import time
import sys
import glob
import netCDF4
import datetime
import os

# for line-based run: define script input manually (comment lines for console run)
#sys.argv = ["", "/scratch3/noemi/wear/swiss_arm/code/output/ARM_run_2022-04-05/scen6/",1982]

print(sys.argv)
# get start time
stm = time.time()

# check if input arguments exist
if len(sys.argv) < 2:
    sys.exit("indicate folder path to analogue dates")
    

start = str(sys.argv[3])
dist = sys.argv[4]

# read folder
dates_ifolder = str(sys.argv[1])  # get first argument passed to script (folder of analogue dates)
print(dates_ifolder)

# create folder for fields
outdir = dates_ifolder + "/ARM_temp_"+  time.strftime("%Y-%m-%d") + "/"
print(outdir)
if not os.path.isdir(outdir):
    try:
        os.mkdir(outdir)
    except OSError:
        print("")
    else:
        print("")

#--------------------------------------------------------------------------------------------------------------------
# READ DATA
# --------------------------------------------------------------------------------------------------------------------
print("read readme")
print(glob.glob(f"{dates_ifolder}*readme*{start}*{dist}*.txt"))

analog_readme = np.array(pd.read_csv(glob.glob(f"{dates_ifolder}*readme*{start}*{dist}*.txt")[0], sep=" "))  # read csv with "readme" in filename
offset = analog_readme[np.argwhere(analog_readme[:,0]=='offset:').item(0),1]
endCal = pd.to_datetime(analog_readme[np.argwhere(analog_readme[:,0]=='endCal:').item(0),1])

print(glob.glob(f'{dates_ifolder}*analog.dates*{start}*{dist}*.txt'))
analog_dates = pd.read_csv(glob.glob(f"{dates_ifolder}*analog.dates*{start}*{dist}*.txt")[0], sep=" ")  # read csv with "analog.dates" in filename

# get target dates and best analogue dates
dates = pd.to_datetime(analog_dates["date"].values)
analog_1_dates = pd.to_datetime(analog_dates["1"], unit='D', origin=pd.Timestamp("1970-01-01"))  # get vector of best analogue dates
analog_1_dates = analog_1_dates.dt.strftime("%Y-%m-%d")

## read offset values
offset_vals = np.array(pd.read_csv("/scratch3/noemi/wear/swiss_arm/code/data_input/runclim_offset_year.txt", sep=" "))  # read csv with "readme" in filename

year= int(sys.argv[2])
print(year)
selyear = dates.year == year
offset_yr = offset_vals[year == offset_vals[:,0],1]

############################################
# read raster data
###########################################
obs_ifolder = "/scratch3/noemi/wear/swiss_arm/code/data_input/"

print("read data")
temp_ifile =  "TabsD_ch01r.swiss.lv95_detrended_19610101-20201231.nc"#"TabsD_detrended_1961-2020_2021-07-21.nc"
temp_obs = xr.open_dataset(obs_ifolder + temp_ifile).TabsD

## change time format
temp_obs["time"] = temp_obs.time.dt.strftime("%Y-%m-%d")

print("create temperature grids from best analogue dates")
if offset == 'TRUE' and dates.year[selyear][1] < 1864:
    temp_analogs = temp_obs.reindex(time=analog_1_dates[selyear]) - offset_yr  
    temp_analogs['time']  = pd.to_datetime(analog_dates["date"][selyear]).values
    temp_analogs = temp_analogs.to_dataset()
    temp_analogs = temp_analogs.rename_vars(name_dict=dict(TabsD="temp"))

else:
    temp_analogs = temp_obs.reindex(time=analog_1_dates[selyear])
    temp_analogs['time']  = pd.to_datetime(analog_dates["date"][selyear]).values
    temp_analogs = temp_analogs.to_dataset()
    temp_analogs = temp_analogs.rename_vars(name_dict=dict(TabsD="temp"))


# write raster data to netCDF files
print("write data to netCDF files")
scal = np.argwhere(analog_readme[:,0]=='startCal:').item(0)
dist = np.argwhere(analog_readme[:,0]=='distance:').item(0)
scen = np.argwhere(analog_readme[:,0]=='scenario:').item(0)
ecal = np.argwhere(analog_readme[:,0]=='endCal:').item(0)

print(scen)
temp_analogs.to_netcdf(outdir + analog_readme[0, 1] + "_temp_analogs_" + str(sys.argv[2]) + "-01-01-" + str(sys.argv[2])  + "-12-31_" + analog_readme[dist, 1] + "_" + analog_readme[scen, 1]  + ".nc", format='NETCDF4')

del temp_analogs, temp_obs

# deseas temp
print("deseas files")
temp_deseas_ifile = "TabsD_ch01r.swiss.lv95_detrended_deseas_19610101-20201231.nc"
temp_deseas_obs = xr.open_dataset(obs_ifolder + temp_deseas_ifile, chunks = dict(time = 4)).TabsD

## change time format
temp_deseas_obs["time"] = temp_deseas_obs.time.dt.strftime("%Y-%m-%d")

print("create temperature grids from best analogue dates")
## it does not really matter for the anomalies, as it would be just plus a constant value, that is then anyway subtracted.
if offset == 'TRUE' and dates.year[selyear][1] < 1864:
    temp_deseas_analogs = temp_deseas_obs.reindex(time=analog_1_dates[selyear])    
    temp_deseas_analogs['time']  = pd.to_datetime(analog_dates["date"][selyear]).values
    temp_deseas_analogs = temp_deseas_analogs.to_dataset()
    temp_deseas_analogs = temp_deseas_analogs.rename_vars(name_dict=dict(TabsD="temp"))

else:
    temp_deseas_analogs = temp_deseas_obs.reindex(time=analog_1_dates[selyear])
    temp_deseas_analogs['time']  = pd.to_datetime(analog_dates["date"][selyear]).values
    temp_deseas_analogs = temp_deseas_analogs.to_dataset()
    temp_deseas_analogs = temp_deseas_analogs.rename_vars(name_dict=dict(TabsD="temp"))

temp_deseas_analogs.to_netcdf(outdir + analog_readme[0, 1] + "_temp_analogs_deseas_"+ str(sys.argv[2]) + "-01-01-" + str(sys.argv[2])  + "-12-31"  +  "_" + analog_readme[dist, 1] + "_" + analog_readme[scen, 1] + ".nc", format='NETCDF4')
    
del temp_deseas_analogs, temp_deseas_obs#, temp_offset_deseas_obs


etm = time.time()
print("temp fields created in: " + str(round((etm-stm)//60)) + " minutes, " + str(round((etm-stm) % 60, 2)) + " seconds")
print("files are in: " + outdir)

### END
