### Calculate error covariance for observations
# three files are created based on different periods.
# however always only two are needed. ref for the evaluation in the reference period and either hist or hist2, depending on the calibration period.

rm(list=ls())

library(dataresqc)
library(geosphere)
library(lubridate)

anomalies <- TRUE # use anomalies or absolute values
variable <- "ta" # change depending on
minlength <- 120 # minimum overlapping period in months

startCal = "1763-01-01"
endCal = "1863-12-31"
detr <- T

args <- commandArgs(TRUE)
if (length(args)>0) {
  scen = args[1]
  startCal = args[2]
  endCal = args[3]
}

### calculate for all_temp always, don't use the scenario
scen <- "all_temp"
print("scenario set to -all_temp- check if detrended station data for all_temp is updates in the data_input folder.")

source("C011_scenarios.R")

station_coords <- XLConnect::readWorksheetFromFile("data_input/arm_stations_inventory_withnbcn.xlsx",sheet=1, colTypes="character",startRow=2)[,c("ID","lat_hist","lon_hist")]
load(paste0("data_input/analogue_stationdata",switch(detr+1,"_","_detrended_refperiod_"),scen,".RData"))
scen_ta <- scenarios[[scen]][grepl("_ta",scenarios[[scen]])]
selection <- paste(scen_ta, collapse = "|")
TOT_ta <- TOT[,c(1,grep(selection,colnames(TOT)))]
n <- ncol(TOT_ta)-1

## Function to calculate anomalies
anom <- function(x, perc=FALSE) {
  xclim <- aggregate(x[,3], list(x[,1]), mean, na.rm=TRUE)
  for (i in 1:12) {
    if (perc) {
      x[x[,1]==i,3] <- 100 * (x[x[,1]==i,3] - xclim$x[xclim[,1]==i]) / xclim$x[xclim[,1]==i]
    } else {
      x[x[,1]==i,3] <- x[x[,1]==i,3] - xclim$x[xclim[,1]==i]
    }
  }
  return(x)
}

## Read all data and metadata into a list
Data_hist  <- list()
for (i in 1:n) {
  Data_hist[[i]] <- list()
  id <- colnames(TOT_ta)[i+1]
  print(id)
  Data_hist[[i]]$coords <- as.numeric(station_coords[station_coords$ID==id,c("lon_hist","lat_hist")])
  yind <- which(year(TOT_ta$date)<=1863& year(TOT_ta$date)>=1763)
  x <- TOT_ta[yind,c("date",id)]
  Data_hist[[i]]$dates <- ISOdate(year(x$date), month(x$date), 1)
  if (anomalies & nrow(x)>=minlength) {
    x <- cbind(month(x$date),year(x$date), x[,2])
    Data_hist[[i]]$ta <- anom(x)[,3]
  } else Data_hist[[i]]$ta <- x$Value
}

names(Data_hist) <- colnames(TOT_ta)[2:ncol(TOT_ta)]

Data_hist2  <- list()
for (i in 1:n) {
  Data_hist2[[i]] <- list()
  id <- colnames(TOT_ta)[i+1]
  print(id)
  Data_hist2[[i]]$coords <- as.numeric(station_coords[station_coords$ID==id,c("lon_hist","lat_hist")])
  yind <- which(year(TOT_ta$date)<=1960 & year(TOT_ta$date)>=1864)
  x <- TOT_ta[yind,c("date",id)]
  Data_hist2[[i]]$dates <- ISOdate(year(x$date), month(x$date), 1)
  if (anomalies & nrow(x)>=minlength) {
    x <- cbind(month(x$date),year(x$date), x[,2]) #x[, c("Month","Year","Value")]
    Data_hist2[[i]]$ta <- anom(x)[,3]
  } else Data_hist2[[i]]$ta <- x$Value
}

names(Data_hist2) <- colnames(TOT_ta)[2:ncol(TOT_ta)]

Data_ref  <- list()
for (i in 1:n) {
  Data_ref[[i]] <- list()
  id <- colnames(TOT_ta)[i+1]
  print(id)
  Data_ref[[i]]$coords <- as.numeric(station_coords[station_coords$ID==id,c("lon_hist","lat_hist")])
  yind <- which(year(TOT_ta$date)<=year(endCal) & year(TOT_ta$date)>=year(startCal))
  x <- TOT_ta[yind,c("date",id)]
  Data_ref[[i]]$dates <- ISOdate(year(x$date), month(x$date), 1)
  if (anomalies & nrow(x)>=minlength) {
    x <- cbind(month(x$date),year(x$date), x[,2]) 
    Data_ref[[i]]$ta <- anom(x)[,3]
  } else Data_ref[[i]]$ta <- x$Value
}

names(Data_ref) <- colnames(TOT_ta)[2:ncol(TOT_ta)]

## Calculate distance and correlation for every pair of stations
cormat_hist <- distmat <- array(dim=c(n,n))
cormat_hist2 <- distmat <- array(dim=c(n,n))
cormat_ref <- distmat <- array(dim=c(n,n))

colnames(cormat_hist) <- colnames(TOT_ta)[2:ncol(TOT_ta)]
rownames(cormat_hist) <- colnames(TOT_ta)[2:ncol(TOT_ta)]
colnames(cormat_hist2) <- colnames(TOT_ta)[2:ncol(TOT_ta)]
rownames(cormat_hist2) <- colnames(TOT_ta)[2:ncol(TOT_ta)]
colnames(cormat_ref) <- colnames(TOT_ta)[2:ncol(TOT_ta)]
rownames(cormat_ref) <- colnames(TOT_ta)[2:ncol(TOT_ta)]

fitvec_hist <- array(dim=n)
fitvec_hist2 <- array(dim=n)
fitvec_ref <- array(dim=n)

for (i in 1:n) {
  message(i)
  for (j in 1:n) {
    if (i != j) {
      distmat[i,j] <- (distGeo(Data_ref[[i]]$coords, Data_ref[[j]]$coords) / 1000)**2
      if (is.na(distmat[i,j])) distmat[i,j] <- 1e+12
      x_hist <- Data_hist[[i]]$ta[which(Data_hist[[i]]$dates %in% Data_hist[[j]]$dates)]
      y_hist <- Data_hist[[j]]$ta[which(Data_hist[[j]]$dates %in% Data_hist[[i]]$dates)]
      
      x_hist2 <- Data_hist2[[i]]$ta[which(Data_hist2[[i]]$dates %in% Data_hist2[[j]]$dates)]
      y_hist2 <- Data_hist2[[j]]$ta[which(Data_hist2[[j]]$dates %in% Data_hist2[[i]]$dates)]
      
      x_ref <- Data_ref[[i]]$ta[which(Data_ref[[i]]$dates %in% Data_ref[[j]]$dates)]
      y_ref <- Data_ref[[j]]$ta[which(Data_ref[[j]]$dates %in% Data_ref[[i]]$dates)]
      
      if (sum(!is.na(x_hist-y_hist)) >= minlength & distmat[i,j]<1000**2) {
        if (sd(x_hist,na.rm=TRUE) > 0 & sd(y_hist,na.rm=TRUE) > 0) {
          cormat_hist[i,j] <- var(x_hist-y_hist, use="pairwise.complete.obs")
        }
      }
      
      if (sum(!is.na(x_hist2-y_hist2)) >= minlength & distmat[i,j]<1000**2) {
        if (sd(x_hist2,na.rm=TRUE) > 0 & sd(y_hist2,na.rm=TRUE) > 0) {
          cormat_hist2[i,j] <- var(x_hist2-y_hist2, use="pairwise.complete.obs")
        }
      }
      
      if (sum(!is.na(x_ref-y_ref)) >= minlength & distmat[i,j]<1000**2) {
        if (sd(x_ref,na.rm=TRUE) > 0 & sd(y_ref,na.rm=TRUE) > 0) {
          cormat_ref[i,j] <- var(x_ref-y_ref, use="pairwise.complete.obs")
        }
      }
    }
  }
  # Linear fit (for error estimation after Wartenburger et al.)
  y_hist <- cormat_hist[i,]
  y_hist2 <- cormat_hist2[i,]
  y_ref <- cormat_ref[i,]
  
  y_hist[which(y_hist>20)] <- NA
  y_hist2[which(y_hist2>20)] <- NA
  y_ref[which(y_ref>20)] <- NA
  
  if (sum(!is.na(y_hist)) >= 7) {
    fitvec_hist[i] <- abs(lm(y_hist ~ distmat[i,])$coefficients[1] / 2)
  }

  if (sum(!is.na(y_hist2)) >= 7) {
    fitvec_hist2[i] <- abs(lm(y_hist2 ~ distmat[i,])$coefficients[1] / 2)
  }
  if (sum(!is.na(y_ref)) >= 7) {
    fitvec_ref[i] <- abs(lm(y_ref ~ distmat[i,])$coefficients[1] / 2)
  }
}

names(fitvec_hist) <- colnames(TOT_ta)[2:ncol(TOT_ta)]
names(fitvec_hist2) <- colnames(TOT_ta)[2:ncol(TOT_ta)]
names(fitvec_ref) <- colnames(TOT_ta)[2:ncol(TOT_ta)]

write.table(fitvec_hist, paste0("data_input/obserror_hist_",scen,".tsv"), row.names=T, quote=FALSE)
write.table(fitvec_hist2, paste0("data_input/obserror_hist2_",scen,".tsv"), row.names=T, quote=FALSE)
write.table(fitvec_ref, paste0("data_input/obserror_ref_",scen,".tsv"), row.names=T, quote=FALSE)

