### 060 Qmap ###
### quantile mapping (for precipitation) 

## data input ###

rm(list=ls())

## different paths for analog recon and ref period recon
#args <- c("output/ARM_run_2022-09-06/scen3/ARM_precip_2022-09-06/2022-09-06_precip_analogs_1961-01-01-2020-12-31_Gower_scen3.nc",
#            "output/ARM_run_2022-09-02/all_hist/ARM_precip_2022-09-02/2022-09-02_precip_analogs_1763-01-01-1863-12-31_Gower_all_hist.nc", 1813, 1863)

# args <- c("output/ARM_run_2022-05-31/scen6/ARM_precip_2022-06-30/2022-05-31_precip_analogs_1961-01-01-2020-12-31_RMSE_scen6.nc", 
#           "output/ARM_run_2022-05-31/scen6/ARM_precip_2022-06-30/2022-05-31_precip_analogs_1864-01-01-1960-12-31_RMSE_scen6.nc", 1864, 1960)

# args <- c("output/ARM_run_2022-06-27/scen4/ARM_precip_2022-06-29/2022-06-27_precip_analogs_1961-01-01-2020-12-31_RMSE_scen4.nc", 
#           "output/ARM_run_2022-06-27/scen4/ARM_precip_2022-06-29/2022-06-27_precip_analogs_1961-01-01-2020-12-31_RMSE_scen4.nc", 1961, 2020)

# args <- c("output/ARM_run_2022-09-05/scen2/ARM_precip_2022-09-05/2022-09-05_precip_analogs_1961-01-01-2020-12-31_Gower_scen2.nc",
#            "output/ARM_run_2022-09-02/all_hist/ARM_precip_2022-09-02/2022-09-02_precip_analogs_1763-01-01-1863-12-31_Gower_all_hist.nc", 1796, 1812)

args <- c("output/ARM_run_2022-09-05/scen1/ARM_precip_2022-09-05/2022-09-05_precip_analogs_1961-01-01-2020-12-31_Gower_scen1.nc", 
          "output/ARM_run_2022-09-02/all_hist/ARM_precip_2022-09-02/2022-09-02_precip_analogs_1763-01-01-1863-12-31_Gower_all_hist.nc", 1763, 1795)

require(raster)
require(qmap)
library(ncdf4);library(lubridate)
library(foreach);library(iterators);library(parallel);library(doParallel)

print(args[1])

qmapset <- list(
  input_raster_obs = "data_input/RhiresD_ch01h.swiss.lv95_1961-01-01-2020-12-31_masked.nc",
  input_raster_rec = args[1],
  input_raster_hist = args[2],
  start = args[3],
  end = args[4],
  method = "QUANT",
  qstep = 0.01,
  wett.day=0.1
)

## create output folder for validation
seldate <- Sys.Date()
subDir <- paste("Qmap",seldate,sep="_")
dirpath <- gsub("ARM_precip.*","",dirname(qmapset$input_raster_rec))
if(!dir.exists(paste0(dirpath,subDir))){dir.create(file.path(dirpath,subDir), showWarnings = F)}
outfolder <- paste0(dirpath,subDir,"/")

# get time
ptm <- proc.time()

#############################################################################################################
### 1. READ DATA
#############################################################################################################
print("read data")

sdate = as.Date("1961-01-01")
edate <- "2020-12-31"

inraster_obs <- qmapset$input_raster_obs
inraster_rec <- qmapset$input_raster_rec
inraster_hist <- qmapset$input_raster_hist

varname_rec <- "precip"
varname_obs <- "RhiresD"

nc_obs <- nc_open(inraster_obs)
nc_rec <- nc_open(inraster_rec)
nc_hist <- nc_open(inraster_hist)

#############################################################################################################
##### define new files ##########
#############################################################################################################

##  create two files
filename_hist <- paste(outfolder,"/",seldate,"_precip_QMAP_",qmapset$start,"-01-01-",qmapset$end,"-12-31.nc",sep="")
filename_ref <- paste(outfolder,"/",seldate,"_precip_QMAP_",sdate,"-",edate,".nc",sep="")

N <- ncvar_get(nc_obs, "N")
E <- ncvar_get(nc_obs, "E")
time_obs <- ncvar_get(nc_obs,"time")
dates_obs <- as.Date(time_obs, origin = "1900-01-01")

time_hist <- ncvar_get(nc_hist,"time")
dates_hist <- as.Date(time_hist, origin = "1763-01-01")

nx <- length(E)
ny <- length(N)
lon1 <- ncdim_def("E", "meter", E)
lat2 <- ncdim_def("N", "meter", N)

time_ref <- ncdim_def("time","days since 1900-01-01", time_obs, unlim=TRUE)
mv <- -999 #missing value to use
var_temp_ref <- ncvar_def("precip", "mm/day", list(lon1, lat2, time_ref), longname="daily precip sums post-processed", mv) 
ncnew_ref <- nc_create(filename_ref, list(var_temp_ref))

time_hist2 <- time_hist[year(dates_hist) >= qmapset$start & year(dates_hist) <= qmapset$end]
time_hist_def <- ncdim_def("time","days since 1763-01-01", time_hist2, unlim=TRUE)
mv <- -999 #missing value to use
var_temp_hist <- ncvar_def("precip", "mm/day", list(lon1, lat2, time_hist_def), longname="daily precip sums post-processed", mv) 
ncnew_hist <- nc_create(filename_hist, list(var_temp_hist))

#############################################################################################################
### 2. DO QUANTILE MAPPING
#############################################################################################################
print("do quantile mapping")

get_5day_factor <- function(dates){
  
  fivedays <- rep(1:73, each = 5)
  doys <- as.numeric(ClimIndVis:::get_date_factors_obj(dates)$jdays)
  fiveday_doys <- fivedays[doys]
  return(fiveday_doys)
  
}

# define qmap functions
qmapping <- function(obs_vp, rec_raw, rec_hist, dates_ref, dates_hist){
  
  fittedvals_rec = rep(NA, length(rec_raw))
  fittedvals_hist = rep(NA, length(rec_hist))
  
  if(any(is.na(rec_raw))){
    return(list(ref = fittedvals_rec, hist = fittedvals_hist))
  }
  else{
    for(wk in 1:73){ ## this is 365 divided by 5
      
      doy_ref <- get_5day_factor(dates_ref)
      doy_hist <- get_5day_factor(dates_hist)
      
      wk_ext <- (wk-2):(wk+2)
      wk_ext[wk_ext < 1] <- wk_ext[wk_ext < 1] + 73
      wk_ext[wk_ext > 73] <- wk_ext[wk_ext  > 73] - 73
      
      sind_ref <- which(doy_ref %in% wk_ext)
      alldoys_ref <- which(doy_ref == wk)
      alldoys_hist <- which(doy_hist == wk)
      
      fitmod <- fitQmap(obs = obs_vp[sind_ref],
                        mod = rec_raw[sind_ref],
                        method = qmapset$method,
                        qstep = qmapset$qstep,
                        wett.day = qmapset$wett.day)
      
      fittedvals_rec[alldoys_ref] <- round(doQmap(rec_raw[alldoys_ref],fitmod),2)
      
      if(any(is.na(rec_hist[alldoys_hist]))){
        alldoys_hist <- alldoys_hist[!is.na(rec_hist[alldoys_hist])]
      } 
      
      fittedvals_hist[alldoys_hist] <- round(doQmap(rec_hist[alldoys_hist],fitmod),2)
    }
    return(list(ref = fittedvals_rec, hist = fittedvals_hist))  
  }
}

chunks <- length(E)
print(chunks)

##### loop of longitudes ########

registerDoParallel(cores=floor(detectCores()*0.15)) 
ii <- 100

startind <- which(dates_hist == paste0(qmapset$start,"-01-01"))
endlen <- length(startind:which(dates_hist == paste0(qmapset$end,"-12-31")))


for(ii in 1:chunks){
  print(paste0("calc chunk: ", ii))
  vals_obs <- ncvar_get(nc_obs, varid = varname_obs, start = c(ii,1,1), count = c(1,-1,-1))
  vals_rec <- ncvar_get(nc_rec, varid = varname_rec, start = c(ii,1,1), count = c(1,-1,-1))
  vals_hist <- ncvar_get(nc_hist, varid = varname_rec, 
                         start = c(ii,1,startind), count = c(1,-1,endlen))
  
  if(!all(is.na(vals_rec))){
    
    obs_list <- split(vals_obs,  seq(nrow(vals_obs)))
    rec_list <- split(vals_rec,  seq(nrow(vals_rec)))
    hist_list <- split(vals_hist,  seq(nrow(vals_hist)))
    
    print("do qmap on chunk")
    tictoc::tic()
    qmap_list <- foreach(a = 1:length(obs_list)) %dopar% {
      qmapping(obs_vp = obs_list[[a]],rec_raw = rec_list[[a]], rec_hist = hist_list[[a]], dates_ref = dates_obs, dates_hist = dates_hist[1:endlen])
    }
    tictoc::toc()
    
    qmap_ref <- t(sapply(qmap_list, function(x) x$ref))
    qmap_hist <- t(sapply(qmap_list, function(x) x$hist))
    
    print("write values")
    ncvar_put(ncnew_ref, var_temp_ref, qmap_ref,  start = c(ii,1,1), count = c(1,-1,-1))
    ncvar_put(ncnew_hist, var_temp_hist, qmap_hist, start = c(ii,1,1), count = c(1,-1,-1))
    
  }
}

## write a file for evaluating the training
nc_close(ncnew_ref)
nc_close(ncnew_hist)

etm = proc.time()

paste("calculation time:",(etm[3]-ptm[3])%/%60,"minutes", round((etm[3]-ptm[3])%%60),"seconds" ,sep=" ")
print("QMAP fit done")
